#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Shreyas Srish <ssrish@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: aci_l3out_floating_svi
short_description: Manage Layer 3 Outside (L3Out) interfaces (l3ext:VirtualLIfP)
description:
- Manage L3Out interfaces on Cisco ACI fabrics.
options:
  tenant:
    description:
    - Name of an existing tenant.
    type: str
    aliases: [ tenant_name ]
    required: true
  l3out:
    description:
    - Name of an existing L3Out.
    type: str
    aliases: [ l3out_name ]
    required: true
  node_profile:
    description:
    - Name of the node profile.
    type: str
    aliases: [ node_profile_name, logical_node ]
    required: true
  interface_profile:
    description:
    - Name of the interface profile.
    type: str
    aliases: [ interface_profile_name, logical_interface ]
    required: true
  pod_id:
    description:
    - Pod ID to build the interface on.
    type: str
  node_id:
    description:
    - Node ID to build the interface on for Port-channels and single ports.
    type: str
  encap:
    description:
    - Encapsulation on the interface (e.g. "vlan-500")
    type: str
  encap_scope:
    description:
    - Encapsulation scope.
    choices: [ vrf, local ]
    type: str
  address:
    description:
    - IP address.
    type: str
    aliases: [ addr, ip_address ]
  mac_address:
    description:
    - The MAC address option of the interface.
    type: str
  link_local_address:
    description:
    - The link local address option of the interface.
    type: str
  mtu:
    description:
    - Interface MTU.
    type: str
  ipv6_dad:
    description:
    - IPv6 Duplicate Address Detection (DAD) feature.
    type: str
    choices: [ enabled, disabled]
  mode:
    description:
    - The mode option for ext-svi interface.
    type: str
    choices: [ regular, native, untagged ]
  dscp:
    description:
    - The target Differentiated Service (DSCP) value.
    - The APIC defaults to C(unspecified) when unset during creation.
    type: str
    choices: [ AF11, AF12, AF13, AF21, AF22, AF23, AF31, AF32, AF33, AF41, AF42, AF43, CS0, CS1, CS2, CS3, CS4, CS5, CS6, CS7, EF, VA, unspecified ]
    aliases: [ target_dscp ]
  external_bridge_group_profile:
    description:
    - The external bridge group profile.
    - Pass "" as the value to remove an existing external bridge group profile (See Examples).
    - This is only supported in APIC v5.0 and above.
    type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  auto_state:
    description:
    - SVI auto state.
    type: str
    choices: [ enabled, disabled ]
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

notes:
- The C(tenant), C(l3out), C(logical_node_profile) and C(logical_interface_profile) must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant), M(cisco.aci.aci_l3out), M(cisco.aci.aci_l3out_logical_node_profile), M(cisco.aci.aci_l3out_logical_interface_profile) \
  modules can be used for this.
seealso:
- module: cisco.aci.aci_tenant
- module: cisco.aci.aci_l3out
- module: cisco.aci.aci_l3out_logical_node_profile
- module: cisco.aci.aci_l3out_logical_interface_profile
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(l3ext:VirtualLIfP)
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Shreyas Srish (@shrsr)
"""

EXAMPLES = r"""
- name: Create a Floating SVI
  cisco.aci.aci_l3out_floating_svi:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    interface_profile: my_interface_profile
    pod_id: 1
    node_id: 201
    encap: vlan-1
    address: 23.45.67.90/24
    external_bridge_group_profile: bridge1
    state: present
  delegate_to: localhost

- name: Remove an external bridge group profile
  cisco.aci.aci_l3out_floating_svi:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    interface_profile: my_interface_profile
    pod_id: 1
    node_id: 201
    encap: vlan-1
    address: 23.45.67.90/24
    external_bridge_group_profile: ""
    state: present
  delegate_to: localhost

- name: Remove a Floating SVI
  cisco.aci.aci_l3out_floating_svi:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    interface_profile: my_interface_profile
    pod_id: 1
    node_id: 201
    encap: vlan-1
    state: absent
  delegate_to: localhost

- name: Query a Floating SVI
  cisco.aci.aci_l3out_floating_svi:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    interface_profile: my_interface_profile
    pod_id: 1
    node_id: 201
    encap: vlan-1
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all the Floating SVIs under an interface profile
  cisco.aci.aci_l3out_floating_svi:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    interface_profile: my_interface_profile
    state: query
  delegate_to: localhost
  register: query_results
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_contract_dscp_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        tenant=dict(type="str", aliases=["tenant_name"], required=True),
        l3out=dict(type="str", aliases=["l3out_name"], required=True),
        node_profile=dict(type="str", aliases=["node_profile_name", "logical_node"], required=True),
        interface_profile=dict(type="str", aliases=["interface_profile_name", "logical_interface"], required=True),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        pod_id=dict(type="str"),
        node_id=dict(type="str"),
        address=dict(type="str", aliases=["addr", "ip_address"]),
        link_local_address=dict(type="str"),
        mac_address=dict(type="str"),
        mtu=dict(type="str"),
        ipv6_dad=dict(type="str", choices=["enabled", "disabled"]),
        mode=dict(type="str", choices=["regular", "native", "untagged"]),
        encap=dict(type="str"),
        encap_scope=dict(type="str", choices=["vrf", "local"]),
        auto_state=dict(type="str", choices=["enabled", "disabled"]),
        external_bridge_group_profile=dict(type="str"),
        dscp=aci_contract_dscp_spec(direction="dscp"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "present", ["pod_id", "node_id", "encap", "address"]],
            ["state", "absent", ["pod_id", "node_id", "encap"]],
        ],
    )

    tenant = module.params.get("tenant")
    l3out = module.params.get("l3out")
    node_profile = module.params.get("node_profile")
    interface_profile = module.params.get("interface_profile")
    state = module.params.get("state")
    pod_id = module.params.get("pod_id")
    node_id = module.params.get("node_id")
    address = module.params.get("address")
    mtu = module.params.get("mtu")
    ipv6_dad = module.params.get("ipv6_dad")
    link_local_address = module.params.get("link_local_address")
    mac_address = module.params.get("mac_address")
    mode = module.params.get("mode")
    encap = module.params.get("encap")
    encap_scope = "ctx" if module.params.get("encap_scope") == "vrf" else module.params.get("encap_scope")
    auto_state = module.params.get("auto_state")
    external_bridge_group_profile = module.params.get("external_bridge_group_profile")

    aci = ACIModule(module)

    node_dn = None
    if pod_id and node_id:
        node_dn = "topology/pod-{0}/node-{1}".format(pod_id, node_id)

    child_classes = []
    if external_bridge_group_profile is not None:
        child_classes.append("l3extBdProfileCont")

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="l3extOut",
            aci_rn="out-{0}".format(l3out),
            module_object=l3out,
            target_filter={"name": l3out},
        ),
        subclass_2=dict(
            aci_class="l3extLNodeP",
            aci_rn="lnodep-{0}".format(node_profile),
            module_object=node_profile,
            target_filter={"name": node_profile},
        ),
        subclass_3=dict(
            aci_class="l3extLIfP",
            aci_rn="lifp-{0}".format(interface_profile),
            module_object=interface_profile,
            target_filter={"name": interface_profile},
        ),
        subclass_4=dict(
            aci_class="l3extVirtualLIfP", aci_rn="vlifp-[{0}]-[{1}]".format(node_dn, encap), module_object=node_dn, target_filter={"nodeDn": node_dn}
        ),
        child_classes=child_classes,
    )

    aci.get_existing()

    if state == "present":
        child_configs = []
        if external_bridge_group_profile is not None:
            if external_bridge_group_profile == "" and isinstance(aci.existing, list) and len(aci.existing) > 0:
                if aci.existing[0].get("l3extVirtualLIfP", {}).get("children") is not None:
                    child_configs.append(
                        dict(
                            l3extBdProfileCont=dict(
                                attributes=dict(status="deleted"),
                            ),
                        )
                    )
            elif external_bridge_group_profile != "":
                child_configs.append(
                    dict(
                        l3extBdProfileCont=dict(
                            attributes=dict(),
                            children=[
                                dict(
                                    l3extRsBdProfile=dict(
                                        attributes=dict(
                                            tDn="uni/tn-{0}/bdprofile-{1}".format(tenant, external_bridge_group_profile),
                                        ),
                                    )
                                )
                            ],
                        )
                    )
                )

        aci.payload(
            aci_class="l3extVirtualLIfP",
            class_config=dict(
                addr=address,
                ipv6Dad=ipv6_dad,
                mtu=mtu,
                ifInstT="ext-svi",
                mode=mode,
                encap=encap,
                encapScope=encap_scope,
                autostate=auto_state,
                llAddr=link_local_address,
                mac=mac_address,
            ),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="l3extVirtualLIfP")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: security_rogue_wireless_containment_status_v1_info
short_description: Information module for Security Rogue Wireless Containment Status V1
description:
- Get Security Rogue Wireless Containment Status V1 by id.
- >
   Intent API to check the wireless rogue access point containment status. The response includes all the details like
   containment status, contained by WLC, containment status of each BSSID etc. This API also includes the information
   of strongest detecting WLC for this rogue access point.
version_added: '6.16.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  macAddress:
    description:
    - MacAddress path parameter. MAC Address of the Wireless Rogue AP.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices WirelessRogueAPContainmentStatusV1
  description: Complete reference of the WirelessRogueAPContainmentStatusV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!wireless-rogue-ap-containment-status
notes:
  - SDK Method used are
    devices.Devices.wireless_rogue_ap_containment_status_v1,

  - Paths used are
    get /dna/intent/api/v1/security/rogue/wireless-containment/status/{macAddress},

"""

EXAMPLES = r"""
- name: Get Security Rogue Wireless Containment Status V1 by id
  cisco.dnac.security_rogue_wireless_containment_status_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    macAddress: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "macAddress": "string",
          "type": 0,
          "classification": "string",
          "containmentStatus": "string",
          "containedByWlcIp": [
            "string"
          ],
          "lastSeen": 0,
          "strongestDetectingWlcIp": "string",
          "lastTaskDetail": {
            "taskId": "string",
            "taskType": "string",
            "taskState": "string",
            "taskStartTime": 0,
            "initiatedOnWlcIp": "string",
            "initiatedOnBssid": [
              "string"
            ]
          },
          "bssidContainmentStatus": [
            {
              "bssid": "string",
              "ssid": "string",
              "radioType": "string",
              "containmentStatus": "string",
              "containedByWlcIp": "string",
              "isAdhoc": true
            }
          ]
        }
      ],
      "version": "string"
    }
"""



# -*- coding: utf-8 -*-

# Copyright: (c) 2022,  Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


class ModuleDocFragment:
    # Standard Tagging related parameters
    DOCUMENTATION = r"""
options:
  tags:
    description:
      - A dictionary representing the tags to be applied to the resource.
      - If the O(tags) parameter is not set then tags will not be modified.
    type: dict
    required: false
    aliases: ['resource_tags']
  purge_tags:
    description:
      - If O(purge_tags=true) and O(tags) is set, existing tags will be purged
        from the resource to match exactly what is defined by O(tags) parameter.
      - If the O(tags) parameter is not set then tags will not be modified, even
        if O(purge_tags=True).
      - Tag keys beginning with V(aws:) are reserved by Amazon and can not be
        modified.  As such they will be ignored for the purposes of the
        O(purge_tags) parameter.  See the Amazon documentation for more information
        U(https://docs.aws.amazon.com/general/latest/gr/aws_tagging.html#tag-conventions).
    type: bool
    default: true
    required: false
"""

    # Modules and Plugins can (currently) use the same fragment
    def __init__(self):
        self.MODULES = self.DOCUMENTATION
        self.PLUGINS = self.DOCUMENTATION



# 2017 Red Hat Inc.
# (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
author:
 - Ansible Networking Team (@ansible-network)
name: persistent
short_description: Use a persistent unix socket for connection
description:
- This is a helper plugin to allow making other connections persistent.
version_added: 1.0.0
extends_documentation_fragment:
- ansible.netcommon.connection_persistent
"""
from ansible.executor.task_executor import start_connection
from ansible.module_utils._text import to_text
from ansible.module_utils.connection import Connection as SocketConnection
from ansible.plugins.connection import ConnectionBase
from ansible.utils.display import Display


display = Display()


class Connection(ConnectionBase):
    """Local based connections"""

    transport = "ansible.netcommon.persistent"
    has_pipelining = False

    def __init__(self, play_context, new_stdin, *args, **kwargs):
        super(Connection, self).__init__(play_context, new_stdin, *args, **kwargs)
        self._task_uuid = to_text(kwargs.get("task_uuid", ""))

    def _connect(self):
        self._connected = True
        return self

    def exec_command(self, cmd, in_data=None, sudoable=True):
        display.vvvv(
            "exec_command(), socket_path=%s" % self.socket_path,
            host=self._play_context.remote_addr,
        )
        connection = SocketConnection(self.socket_path)
        out = connection.exec_command(cmd, in_data=in_data, sudoable=sudoable)
        return 0, out, ""

    def put_file(self, in_path, out_path):
        pass

    def fetch_file(self, in_path, out_path):
        pass

    def close(self):
        self._connected = False

    def run(self):
        """Returns the path of the persistent connection socket.

        Attempts to ensure (within playcontext.timeout seconds) that the
        socket path exists. If the path exists (or the timeout has expired),
        returns the socket path.
        """
        display.vvvv(
            "starting connection from persistent connection plugin",
            host=self._play_context.remote_addr,
        )
        display.deprecated(
            msg="support for connection local has been deprecated",
            date="2027-01-01",
            collection_name="ansible.netcommon",
        )
        variables = {"ansible_command_timeout": self.get_option("persistent_command_timeout")}
        socket_path = start_connection(self._play_context, variables, self._task_uuid)
        display.vvvv(
            "local domain socket path is %s" % socket_path,
            host=self._play_context.remote_addr,
        )
        setattr(self, "_socket_path", socket_path)
        return socket_path



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: pnp_server_profile_update
short_description: Resource module for Pnp Server Profile Update
description:
- This module represents an alias of the module pnp_server_profile_update_v1
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  ccoUser:
    description: Cco User.
    type: str
  profile:
    description: Pnp Server Profile Update's profile.
    suboptions:
      addressFqdn:
        description: Required when cluster is configured with fully qualified domain
          name (FQDN).
        type: str
      addressIpV4:
        description: Required when cluster is configured with IPv4.
        type: str
      addressIpV6:
        description: Required when cluster is configured with IPv6.
        type: str
      cert:
        description: Cert.
        type: str
      makeDefault:
        description: Make Default.
        type: bool
      name:
        description: Name.
        type: str
      port:
        description: Port.
        type: float
      profileId:
        description: Profile Id.
        type: str
      proxy:
        description: Proxy.
        type: bool
    type: dict
  smartAccountId:
    description: Smart Account Id.
    type: str
  virtualAccountId:
    description: Virtual Account Id.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Device Onboarding (PnP) UpdatePnPServerProfileV1
  description: Complete reference of the UpdatePnPServerProfileV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-pn-p-server-profile
notes:
  - SDK Method used are
    device_onboarding_pnp.DeviceOnboardingPnp.update_pnp_server_profile_v1,

  - Paths used are
    put /dna/intent/api/v1/onboarding/pnp-settings/savacct,
  - It should be noted that this module is an alias of pnp_server_profile_update_v1

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.pnp_server_profile_update:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    ccoUser: string
    profile:
      addressFqdn: string
      addressIpV4: string
      addressIpV6: string
      cert: string
      makeDefault: true
      name: string
      port: 0
      profileId: string
      proxy: true
    smartAccountId: string
    virtualAccountId: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "virtualAccountId": "string",
      "autoSyncPeriod": 0,
      "syncResultStr": "string",
      "profile": {
        "proxy": true,
        "makeDefault": true,
        "port": 0,
        "profileId": "string",
        "name": "string",
        "addressIpV4": "string",
        "cert": "string",
        "addressFqdn": "string"
      },
      "ccoUser": "string",
      "syncResult": {
        "syncList": [
          {
            "syncType": "string",
            "deviceSnList": [
              "string"
            ]
          }
        ],
        "syncMsg": "string"
      },
      "token": "string",
      "syncStartTime": 0,
      "lastSync": 0,
      "tenantId": "string",
      "smartAccountId": "string",
      "expiry": 0,
      "syncStatus": "string"
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2019 Zim Kalinowski, (@zikalino)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_devtestlabpolicy
version_added: "0.1.2"
short_description: Manage Azure Policy instance
description:
    - Create, update and delete instance of Azure Policy.

options:
    resource_group:
        description:
            - The name of the resource group.
        required: True
        type: str
    lab_name:
        description:
            - The name of the lab.
        required: True
        type: str
    policy_set_name:
        description:
            - The name of the policy set.
        required: True
        type: str
    name:
        description:
            - The name of the policy.
        required: True
        type: str
    description:
        description:
            - The description of the policy.
        type: str
    fact_name:
        description:
            - The fact name of the policy (e.g. C(lab_vm_count), C(lab_vm_size)), MaxVmsAllowedPerLab, etc.
        type: str
        choices:
            - 'user_owned_lab_vm_count'
            - 'user_owned_lab_premium_vm_count'
            - 'lab_vm_count'
            - 'lab_premium_vm_count'
            - 'lab_vm_size'
            - 'gallery_image'
            - 'user_owned_lab_vm_count_in_subnet'
            - 'lab_target_cost'
    threshold:
        description:
            - The threshold of the policy (it could be either a maximum value or a list of allowed values).
        type: raw
    state:
        description:
            - Assert the state of the Policy.
            - Use C(present) to create or update an Policy and C(absent) to delete it.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Zim Kalinowski (@zikalino)

'''

EXAMPLES = '''
- name: Create DevTest Lab Policy
  azure_rm_devtestlabpolicy:
    resource_group: myResourceGroup
    lab_name: myLab
    policy_set_name: myPolicySet
    name: myPolicy
    fact_name: user_owned_lab_vm_count
    threshold: 5
'''

RETURN = '''
id:
    description:
        - The identifier of the resource.
    returned: always
    type: str
    sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourcegroups/myResourceGroup/providers/microsoft.devtestlab/labs/myLab/policySets/
             myPolicySet/policies/myPolicy"

'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
from ansible.module_utils.common.dict_transformations import _snake_to_camel

try:
    from azure.core.polling import LROPoller
    from azure.core.exceptions import ResourceNotFoundError
    from azure.mgmt.devtestlabs import DevTestLabsClient
except ImportError:
    # This is handled in azure_rm_common
    pass


class Actions:
    NoAction, Create, Update, Delete = range(4)


class AzureRMDtlPolicy(AzureRMModuleBase):
    """Configuration class for an Azure RM Policy resource"""

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            lab_name=dict(
                type='str',
                required=True
            ),
            policy_set_name=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            description=dict(
                type='str'
            ),
            fact_name=dict(
                type='str',
                choices=['user_owned_lab_vm_count',
                         'user_owned_lab_premium_vm_count',
                         'lab_vm_count',
                         'lab_premium_vm_count',
                         'lab_vm_size',
                         'gallery_image',
                         'user_owned_lab_vm_count_in_subnet',
                         'lab_target_cost']
            ),
            threshold=dict(
                type='raw'
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        self.resource_group = None
        self.lab_name = None
        self.policy_set_name = None
        self.name = None
        self.policy = dict()

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.state = None
        self.to_do = Actions.NoAction

        required_if = [
            ('state', 'present', ['threshold', 'fact_name'])
        ]

        super(AzureRMDtlPolicy, self).__init__(derived_arg_spec=self.module_arg_spec,
                                               supports_check_mode=True,
                                               supports_tags=True,
                                               required_if=required_if)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                self.policy[key] = kwargs[key]

        if self.state == 'present':
            self.policy['status'] = 'Enabled'
            dict_camelize(self.policy, ['fact_name'], True)
            if isinstance(self.policy['threshold'], list):
                self.policy['evaluator_type'] = 'AllowedValuesPolicy'
            else:
                self.policy['evaluator_type'] = 'MaxValuePolicy'

        response = None

        self.mgmt_client = self.get_mgmt_svc_client(DevTestLabsClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)

        resource_group = self.get_resource_group(self.resource_group)

        old_response = self.get_policy()

        if not old_response:
            self.log("Policy instance doesn't exist")
            if self.state == 'absent':
                self.log("Old instance didn't exist")
            else:
                self.to_do = Actions.Create
        else:
            self.log("Policy instance already exists")
            if self.state == 'absent':
                self.to_do = Actions.Delete
            elif self.state == 'present':
                if (not default_compare(self.policy, old_response, '', self.results)):
                    self.to_do = Actions.Update

        if (self.to_do == Actions.Create) or (self.to_do == Actions.Update):
            self.log("Need to Create / Update the Policy instance")

            if self.check_mode:
                self.results['changed'] = True
                return self.results

            response = self.create_update_policy()

            self.results['changed'] = True
            self.log("Creation / Update done")
        elif self.to_do == Actions.Delete:
            self.log("Policy instance deleted")
            self.results['changed'] = True

            if self.check_mode:
                return self.results

            self.delete_policy()
            # This currently doesnt' work as there is a bug in SDK / Service
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
        else:
            self.log("Policy instance unchanged")
            self.results['changed'] = False
            response = old_response

        if self.state == 'present':
            self.results.update({
                'id': response.get('id', None),
                'status': response.get('status', None)
            })
        return self.results

    def create_update_policy(self):
        '''
        Creates or updates Policy with the specified configuration.

        :return: deserialized Policy instance state dictionary
        '''
        self.log("Creating / Updating the Policy instance {0}".format(self.name))

        try:
            response = self.mgmt_client.policies.create_or_update(resource_group_name=self.resource_group,
                                                                  lab_name=self.lab_name,
                                                                  policy_set_name=self.policy_set_name,
                                                                  name=self.name,
                                                                  policy=self.policy)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)

        except Exception as exc:
            self.log('Error attempting to create the Policy instance.')
            self.fail("Error creating the Policy instance: {0}".format(str(exc)))
        return response.as_dict()

    def delete_policy(self):
        '''
        Deletes specified Policy instance in the specified subscription and resource group.

        :return: True
        '''
        self.log("Deleting the Policy instance {0}".format(self.name))
        try:
            response = self.mgmt_client.policies.delete(resource_group_name=self.resource_group,
                                                        lab_name=self.lab_name,
                                                        policy_set_name=self.policy_set_name,
                                                        name=self.name)
        except Exception as e:
            self.log('Error attempting to delete the Policy instance.')
            self.fail("Error deleting the Policy instance: {0}".format(str(e)))

        return True

    def get_policy(self):
        '''
        Gets the properties of the specified Policy.

        :return: deserialized Policy instance state dictionary
        '''
        self.log("Checking if the Policy instance {0} is present".format(self.name))
        found = False
        try:
            response = self.mgmt_client.policies.get(resource_group_name=self.resource_group,
                                                     lab_name=self.lab_name,
                                                     policy_set_name=self.policy_set_name,
                                                     name=self.name)
            found = True
            self.log("Response : {0}".format(response))
            self.log("Policy instance : {0} found".format(response.name))
        except ResourceNotFoundError as e:
            self.log('Did not find the Policy instance.')
        if found is True:
            return response.as_dict()

        return False


def default_compare(new, old, path, result):
    if new is None:
        return True
    elif isinstance(new, dict):
        if not isinstance(old, dict):
            result['compare'] = 'changed [' + path + '] old dict is null'
            return False
        for k in new.keys():
            if not default_compare(new.get(k), old.get(k, None), path + '/' + k, result):
                return False
        return True
    elif isinstance(new, list):
        if not isinstance(old, list) or len(new) != len(old):
            result['compare'] = 'changed [' + path + '] length is different or null'
            return False
        if isinstance(old[0], dict):
            key = None
            if 'id' in old[0] and 'id' in new[0]:
                key = 'id'
            elif 'name' in old[0] and 'name' in new[0]:
                key = 'name'
            else:
                key = list(old[0])[0]
            new = sorted(new, key=lambda x: x.get(key, None))
            old = sorted(old, key=lambda x: x.get(key, None))
        else:
            new = sorted(new)
            old = sorted(old)
        for i in range(len(new)):
            if not default_compare(new[i], old[i], path + '/*', result):
                return False
        return True
    else:
        if path == '/location':
            new = new.replace(' ', '').lower()
            old = new.replace(' ', '').lower()
        if str(new) == str(old):
            return True
        else:
            result['compare'] = 'changed [' + path + '] ' + str(new) + ' != ' + str(old)
            return False


def dict_camelize(d, path, camelize_first):
    if isinstance(d, list):
        for i in range(len(d)):
            dict_camelize(d[i], path, camelize_first)
    elif isinstance(d, dict):
        if len(path) == 1:
            old_value = d.get(path[0], None)
            if old_value is not None:
                d[path[0]] = _snake_to_camel(old_value, camelize_first)
        else:
            sd = d.get(path[0], None)
            if sd is not None:
                dict_camelize(sd, path[1:], camelize_first)


def dict_map(d, path, map):
    if isinstance(d, list):
        for i in range(len(d)):
            dict_map(d[i], path, map)
    elif isinstance(d, dict):
        if len(path) == 1:
            old_value = d.get(path[0], None)
            if old_value is not None:
                d[path[0]] = map.get(old_value, old_value)
        else:
            sd = d.get(path[0], None)
            if sd is not None:
                dict_map(sd, path[1:], map)


def main():
    """Main execution"""
    AzureRMDtlPolicy()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_devices_summary_analytics_v1
short_description: Resource module for Network Devices Summary Analytics V1
description:
- Manage operation create of the resource Network Devices Summary Analytics V1.
- >
   Gets the summary analytics data related to network devices based on the provided input data. This endpoint helps
   to obtain the consolidated insights into the performance and status of the monitored network devices. For detailed
   information about the usage of the API, please refer to the Open API specification document - https
   //github.com/cisco-en-programmability/catalyst-center-api-specs/blob/main/Assurance/CE_Cat_Center_Org-
   AssuranceNetworkDevices-1.0.2-resolved.yaml.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  aggregateAttributes:
    description: Network Devices Summary Analytics's aggregateAttributes.
    elements: dict
    suboptions:
      function:
        description: Function.
        type: str
      name:
        description: Name.
        type: str
    type: list
  attributes:
    description: Attributes.
    elements: str
    type: list
  endTime:
    description: End Time.
    type: int
  filters:
    description: Network Devices Summary Analytics's filters.
    elements: dict
    suboptions:
      key:
        description: Key.
        type: str
      operator:
        description: Operator.
        type: str
      value:
        description: Value.
        type: str
    type: list
  groupBy:
    description: Group By.
    elements: str
    type: list
  page:
    description: Network Devices Summary Analytics's page.
    suboptions:
      limit:
        description: Limit.
        type: int
      offset:
        description: Offset.
        type: int
      sortBy:
        description: Network Devices Summary Analytics's sortBy.
        elements: dict
        suboptions:
          name:
            description: Name.
            type: str
          order:
            description: Order.
            type: str
        type: list
    type: dict
  startTime:
    description: Start Time.
    type: int
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices GetsTheSummaryAnalyticsDataRelatedToNetworkDevicesV1
  description: Complete reference of the GetsTheSummaryAnalyticsDataRelatedToNetworkDevicesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!gets-the-summary-analytics-data-related-to-network-devices
notes:
  - SDK Method used are
    devices.Devices.gets_the_summary_analytics_data_related_to_network_devices_v1,

  - Paths used are
    post /dna/data/api/v1/networkDevices/summaryAnalytics,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.network_devices_summary_analytics_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    aggregateAttributes:
    - function: string
      name: string
    attributes:
    - string
    endTime: 0
    filters:
    - key: string
      operator: string
      value: string
    groupBy:
    - string
    page:
      limit: 0
      offset: 0
      sortBy:
      - name: string
        order: string
    startTime: 0

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "attributes": [
          {}
        ],
        "aggregateAttributes": [
          {}
        ],
        "groups": [
          {
            "id": "string",
            "attributes": [
              {
                "name": "string",
                "value": "string"
              }
            ],
            "aggregateAttributes": [
              {
                "name": "string",
                "function": "string",
                "value": 0
              }
            ]
          }
        ]
      },
      "page": [
        {
          "limit": 0,
          "offset": 0,
          "count": 0,
          "sortBy": [
            {
              "name": "string",
              "order": "string"
            }
          ]
        }
      ]
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ec2_vpc_net_info
version_added: 1.0.0
short_description: Gather information about EC2 VPCs in AWS
description:
    - Gather information about EC2 VPCs in AWS.
author: "Rob White (@wimnat)"
options:
  vpc_ids:
    description:
      - A list of VPC IDs that exist in your account.
    type: list
    elements: str
    default: []
  filters:
    description:
      - A dict of filters to apply. Each dict item consists of a filter key and a filter value.
        See U(https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeVpcs.html) for possible filters.
    type: dict
    default: {}
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

# Gather information about all VPCs
- amazon.aws.ec2_vpc_net_info:

# Gather information about a particular VPC using VPC ID
- amazon.aws.ec2_vpc_net_info:
    vpc_ids: vpc-00112233

# Gather information about any VPC with a tag key Name and value Example
- amazon.aws.ec2_vpc_net_info:
    filters:
      "tag:Name": Example
"""

RETURN = r"""
vpcs:
    description: Returns an array of complex objects as described below.
    returned: success
    type: complex
    contains:
        id:
            description: The ID of the VPC (for backwards compatibility).
            returned: always
            type: str
        vpc_id:
            description: The ID of the VPC.
            returned: always
            type: str
        state:
            description: The state of the VPC.
            returned: always
            type: str
        tags:
            description: A dict of tags associated with the VPC.
            returned: always
            type: dict
        instance_tenancy:
            description: The instance tenancy setting for the VPC.
            returned: always
            type: str
        is_default:
            description: True if this is the default VPC for account.
            returned: always
            type: bool
        cidr_block:
            description: The IPv4 CIDR block assigned to the VPC.
            returned: always
            type: str
        enable_dns_hostnames:
            description: True/False depending on attribute setting for DNS hostnames support.
            returned: always
            type: bool
        enable_dns_support:
            description: True/False depending on attribute setting for DNS support.
            returned: always
            type: bool
        cidr_block_association_set:
            description: An array of IPv4 cidr block association set information.
            returned: always
            type: complex
            contains:
                association_id:
                    description: The association ID.
                    returned: always
                    type: str
                cidr_block:
                    description: The IPv4 CIDR block that is associated with the VPC.
                    returned: always
                    type: str
                cidr_block_state:
                    description: A hash/dict that contains a single item. The state of the cidr block association.
                    returned: always
                    type: dict
                    contains:
                        state:
                            description: The CIDR block association state.
                            returned: always
                            type: str
        ipv6_cidr_block_association_set:
            description: An array of IPv6 cidr block association set information.
            returned: always
            type: complex
            contains:
                association_id:
                    description: The association ID.
                    returned: always
                    type: str
                ipv6_cidr_block:
                    description: The IPv6 CIDR block that is associated with the VPC.
                    returned: always
                    type: str
                ipv6_cidr_block_state:
                    description: A hash/dict that contains a single item. The state of the cidr block association.
                    returned: always
                    type: dict
                    contains:
                        state:
                            description: The CIDR block association state.
                            returned: always
                            type: str
        owner_id:
            description: The AWS account which owns the VPC.
            returned: always
            type: str
            sample: 123456789012
        dhcp_options_id:
            description: The ID of the DHCP options associated with this VPC.
            returned: always
            type: str
            sample: dopt-12345678
"""

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.ec2 import AnsibleEC2Error
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_vpc_attribute
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_vpcs
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list


def list_vpcs(connection, module: AnsibleAWSModule) -> None:
    """
    Describe VPCs.

    connection  : boto3 client connection object
    module  : AnsibleAWSModule object
    """
    # collect parameters
    filters = ansible_dict_to_boto3_filter_list(module.params.get("filters"))
    vpc_ids = module.params.get("vpc_ids")

    # init empty list for return vars
    vpc_info = list()

    # Get the basic VPC info
    try:
        vpcs = describe_vpcs(connection, VpcIds=vpc_ids, Filters=filters)
    except AnsibleEC2Error as e:
        module.fail_json_aws(e, msg=f"Unable to describe VPCs {vpc_ids}")

    # Loop through the results and add the other VPC attributes we gathered
    for vpc in vpcs:
        error_message = "Unable to describe VPC attribute {0} on VPC {1}"
        vpc_attributes = {}
        for attribute in ("enableDnsSupport", "enableDnsHostnames"):
            try:
                vpc_attributes[attribute] = describe_vpc_attribute(connection, vpc_id=vpc["VpcId"], attribute=attribute)
                if not vpc_attributes[attribute]:
                    module.warn(error_message.format(attribute, vpc["VpcId"]))
            except AnsibleEC2Error as e:
                module.fail_json_aws(e, msg=error_message.format(attribute, vpc))

        # add the two DNS attributes
        if vpc_attributes["enableDnsSupport"]:
            vpc["EnableDnsSupport"] = vpc_attributes["enableDnsSupport"]["EnableDnsSupport"].get("Value")
        if vpc_attributes["enableDnsHostnames"]:
            vpc["EnableDnsHostnames"] = vpc_attributes["enableDnsHostnames"]["EnableDnsHostnames"].get("Value")
        # for backwards compatibility
        vpc["id"] = vpc["VpcId"]
        vpc_info.append(camel_dict_to_snake_dict(vpc))
        # convert tag list to ansible dict
        vpc_info[-1]["tags"] = boto3_tag_list_to_ansible_dict(vpc.get("Tags", []))

    module.exit_json(vpcs=vpc_info)


def main() -> None:
    argument_spec = dict(
        vpc_ids=dict(type="list", elements="str", default=[]),
        filters=dict(type="dict", default={}),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)
    connection = module.client("ec2")

    list_vpcs(connection, module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sites_device_credentials_apply_v1
short_description: Resource module for Sites Device Credentials Apply V1
description:
- Manage operation create of the resource Sites Device Credentials Apply V1.
- >
   When sync is triggered at a site with the credential that are associated to the same site, network devices in
   impacted sites child sites which are inheriting the credential get managed in inventory with the associated site
   credential. Credential gets configured on network devices before these get managed in inventory. Please make a
   note that cli credential wouldn't be configured on AAA authenticated devices but they just get managed with the
   associated site cli credential.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  deviceCredentialId:
    description: It must be cli/snmpV2Read/snmpV2Write/snmpV3 Id.
    type: str
  siteId:
    description: Site Id.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Network Settings SyncNetworkDevicesCredentialV1
  description: Complete reference of the SyncNetworkDevicesCredentialV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!sync-network-devices-credential
notes:
  - SDK Method used are
    network_settings.NetworkSettings.sync_network_devices_credential_v1,

  - Paths used are
    post /dna/intent/api/v1/sites/deviceCredentials/apply,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.sites_device_credentials_apply_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    deviceCredentialId: string
    siteId: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": {
        "url": "string",
        "taskId": "string"
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: endpoint_analytics_profiling_rules
short_description: Resource module for Endpoint Analytics Profiling Rules
description:
- This module represents an alias of the module endpoint_analytics_profiling_rules_v1
version_added: '6.16.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  clusterId:
    description: Unique identifier for ML cluster. Only applicable for 'ML Rule'.
    type: str
  conditionGroups:
    description: Endpoint Analytics Profiling Rules's conditionGroups.
    suboptions:
      condition:
        description: Endpoint Analytics Profiling Rules's condition.
        suboptions:
          attribute:
            description: Endpoint Analytics Profiling Rules's attribute.
            type: str
          attributeDictionary:
            description: Endpoint Analytics Profiling Rules's attributeDictionary.
            type: str
          operator:
            description: Endpoint Analytics Profiling Rules's operator.
            type: str
          value:
            description: Endpoint Analytics Profiling Rules's value.
            type: str
        type: dict
      conditionGroup:
        description: Endpoint Analytics Profiling Rules's conditionGroup.
        elements: str
        type: list
      operator:
        description: Endpoint Analytics Profiling Rules's operator.
        type: str
      type:
        description: Endpoint Analytics Profiling Rules's type.
        type: str
    type: dict
  isDeleted:
    description: Flag to indicate whether the rule was deleted.
    type: bool
  lastModifiedBy:
    description: User that last modified the rule. It is read-only, and is ignored if
      provided as part of input request.
    type: str
  lastModifiedOn:
    description: Timestamp (in epoch milliseconds) of last modification. It is read-only,
      and is ignored if provided as part of input request.
    type: int
  pluginId:
    description: Plugin for the rule. Only applicable for 'Cisco Default' rules.
    type: str
  rejected:
    description: Flag to indicate whether rule has been rejected by user or not. Only
      applicable for 'ML Rule'.
    type: bool
  result:
    description: Endpoint Analytics Profiling Rules's result.
    suboptions:
      deviceType:
        description: List of device types determined by the current rule.
        elements: str
        type: list
      hardwareManufacturer:
        description: List of hardware manufacturers determined by the current rule.
        elements: str
        type: list
      hardwareModel:
        description: List of hardware models determined by the current rule.
        elements: str
        type: list
      operatingSystem:
        description: List of operating systems determined by the current rule.
        elements: str
        type: list
    type: dict
  ruleId:
    description: Unique identifier for the rule. This is normally generated by the system,
      and client does not need to provide it for rules that need to be newly created.
    type: str
  ruleName:
    description: Human readable name for the rule.
    type: str
  rulePriority:
    description: Priority for the rule.
    type: int
  ruleType:
    description: Type of the rule.
    type: str
  ruleVersion:
    description: Version of the rule.
    type: int
  sourcePriority:
    description: Source priority for the rule.
    type: int
  usedAttributes:
    description: List of attributes used in the rule. Only applicable for 'Cisco Default'
      rules.
    elements: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
notes:
  - SDK Method used are
    ai_endpoint_analytics.AIEndpointAnalytics.create_a_profiling_rule_v1,
    ai_endpoint_analytics.AIEndpointAnalytics.delete_an_existing_profiling_rule_v1,
    ai_endpoint_analytics.AIEndpointAnalytics.update_an_existing_profiling_rule_v1,

  - Paths used are
    post /dna/intent/api/v1/endpoint-analytics/profiling-rules,
    delete /dna/intent/api/v1/endpoint-analytics/profiling-rules/{ruleId},
    put /dna/intent/api/v1/endpoint-analytics/profiling-rules/{ruleId},
  - It should be noted that this module is an alias of endpoint_analytics_profiling_rules_v1

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.endpoint_analytics_profiling_rules:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    clusterId: string
    conditionGroups:
      condition:
        attribute: string
        attributeDictionary: string
        operator: string
        value: string
      conditionGroup:
      - string
      operator: string
      type: string
    isDeleted: true
    lastModifiedBy: string
    lastModifiedOn: 0
    pluginId: string
    rejected: true
    result:
      deviceType:
      - string
      hardwareManufacturer:
      - string
      hardwareModel:
      - string
      operatingSystem:
      - string
    ruleId: string
    ruleName: string
    rulePriority: 0
    ruleType: string
    ruleVersion: 0
    sourcePriority: 0
    usedAttributes:
    - string

- name: Update by id
  cisco.dnac.endpoint_analytics_profiling_rules:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    clusterId: string
    conditionGroups:
      condition:
        attribute: string
        attributeDictionary: string
        operator: string
        value: string
      conditionGroup:
      - string
      operator: string
      type: string
    isDeleted: true
    lastModifiedBy: string
    lastModifiedOn: 0
    pluginId: string
    rejected: true
    result:
      deviceType:
      - string
      hardwareManufacturer:
      - string
      hardwareModel:
      - string
      operatingSystem:
      - string
    ruleId: string
    ruleName: string
    rulePriority: 0
    ruleType: string
    ruleVersion: 0
    sourcePriority: 0
    usedAttributes:
    - string

- name: Delete by id
  cisco.dnac.endpoint_analytics_profiling_rules:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    ruleId: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "link": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: floors_v2
short_description: Resource module for Floors V2
description:
- Manage operations create, update and delete of the resource Floors V2.
- Create a floor in the network hierarchy under building.
- Deletes a floor from the network hierarchy. This operations fails if there are any devices assigned to this floor.
- Updates a floor in the network hierarchy.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  floorNumber:
    description: Floor number.
    type: int
  height:
    description: Floor height. Example 10.1.
    type: float
  id:
    description: Id path parameter. Floor Id.
    type: str
  length:
    description: Floor length. Example 110.3.
    type: float
  name:
    description: Floor name.
    type: str
  parentId:
    description: Parent Id.
    type: str
  rfModel:
    description: RF Model.
    type: str
  unitsOfMeasure:
    description: Units Of Measure.
    type: str
  width:
    description: Floor width. Example 100.5.
    type: float
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Site Design CreatesAFloorV2
  description: Complete reference of the CreatesAFloorV2 API.
  link: https://developer.cisco.com/docs/dna-center/#!creates-a-floor-v-2
- name: Cisco DNA Center documentation for Site Design DeletesAFloorV2
  description: Complete reference of the DeletesAFloorV2 API.
  link: https://developer.cisco.com/docs/dna-center/#!deletes-a-floor-v-2
- name: Cisco DNA Center documentation for Site Design UpdatesAFloorV2
  description: Complete reference of the UpdatesAFloorV2 API.
  link: https://developer.cisco.com/docs/dna-center/#!updates-a-floor-v-2
notes:
  - SDK Method used are
    site_design.SiteDesign.creates_a_floor_v2,
    site_design.SiteDesign.deletes_a_floor_v2,
    site_design.SiteDesign.updates_a_floor_v2,

  - Paths used are
    post /dna/intent/api/v2/floors,
    delete /dna/intent/api/v2/floors/{id},
    put /dna/intent/api/v2/floors/{id},

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.floors_v2:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    floorNumber: 0
    height: 0
    length: 0
    name: string
    parentId: string
    rfModel: string
    unitsOfMeasure: string
    width: 0

- name: Update by id
  cisco.dnac.floors_v2:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    floorNumber: 0
    height: 0
    id: string
    length: 0
    name: string
    parentId: string
    rfModel: string
    unitsOfMeasure: string
    width: 0

- name: Delete by id
  cisco.dnac.floors_v2:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    id: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": {
        "url": "string",
        "taskId": "string"
      }
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2017 Yuwei Zhou, <yuwzho@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_automationaccount_info
version_added: '0.1.2'
short_description: Get Azure automation account facts
description:
    - Get facts of automation account.

options:
    resource_group:
        description:
            - The name of the resource group.
        type: str
        required: True
    name:
        description:
            - The name of the automation account.
        type: str
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str
    list_statistics:
        description:
            - List statistics details for a automation account.
            - Note this will cost network overhead, suggest only used when I(name) set.
        type: bool
    list_usages:
        description:
            - List usage details for a automation account.
            - Note this will cost network overhead, suggest only used when I(name) set.
        type: bool
    list_keys:
        description:
            - List keys for a automation account.
            - Note this will cost network overhead, suggest only used when I(name) set.
        type: bool

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Yuwei Zhou (@yuwzho)

'''

EXAMPLES = '''
- name: Get details of an automation account
  azure_rm_automationaccount_info:
    name: Testing
    resource_group: myResourceGroup
    list_statistics: true
    list_usages: true
    list_keys: true
    tags:
      - key
      - key:value

- name: List automation account in a resource group
  azure_rm_automationaccount_info:
    resource_group: myResourceGroup

- name: List automation account in a resource group
  azure_rm_automationaccount_info:
'''

RETURN = '''
automation_accounts:
    description:
        - List of automation account dicts.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID.
            type: str
            returned: always
            sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups
                     /myResourceGroup/providers/Microsoft.Automation/automationAccounts/Testing"
        resource_group:
            description:
                - Resource group name.
            type: str
            returned: always
            sample: myResourceGroup
        name:
            description:
                - Resource name.
            type: str
            returned: always
            sample: Testing
        location:
            description:
                - Resource location.
            type: str
            returned: always
            sample: eastus
        creation_time:
            description:
                - Resource creation date time.
            type: str
            returned: always
            sample: "2019-04-26T02:55:16.500Z"
        last_modified_time:
            description:
                - Resource last modified date time.
            type: str
            returned: always
            sample: "2019-04-26T02:55:16.500Z"
        state:
            description:
                - Resource state.
            type: str
            returned: always
            sample: ok
        keys:
            description:
                - Resource keys.
            type: complex
            returned: always
            contains:
                key_name:
                    description:
                        - Name of the key.
                    type: str
                    returned: always
                    sample: Primary
                permissions:
                    description:
                        - Permission of the key.
                    type: str
                    returned: always
                    sample: Full
                value:
                    description:
                        - Value of the key.
                    type: str
                    returned: always
                    sample: "MbepKTO6IyGwml0GaKBkKN"
        statistics:
            description:
                - Resource statistics.
            type: complex
            returned: always
            contains:
                counter_property:
                    description:
                        - Property value of the statistic.
                    type: str
                    returned: always
                    sample: New
                counter_value:
                    description:
                        - Value of the statistic.
                    type: int
                    returned: always
                    sample: 0
                end_time:
                    description:
                        - EndTime of the statistic.
                    type: str
                    returned: always
                    sample: "2019-04-26T06:29:43.587518Z"
                id:
                    description:
                        - ID of the statistic.
                    type: str
                    returned: always
                    sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups
                             /myResourceGroup/providers/Microsoft.Automation/automationAccounts/Testing/statistics/New"
                start_time:
                    description:
                        - StartTime of the statistic.
                    type: str
                    returned: always
                    sample: "2019-04-26T06:29:43.587518Z"
        usages:
            description:
                - Resource usages.
            type: complex
            returned: always
            contains:
                current_value:
                    description:
                        - Current usage.
                    type: float
                    returned: always
                    sample: 0.0
                limit:
                    description:
                        - Max limit, C(-1) for unlimited.
                    type: int
                    returned: always
                    sample: -1
                name:
                    description:
                        - Usage counter name.
                    type: complex
                    returned: always
                    contains:
                        localized_value:
                            description:
                                - Localized name.
                            type: str
                            returned: always
                            sample: "SubscriptionUsage"
                        value:
                            description:
                                - Name value.
                            type: str
                            returned: always
                            sample: "SubscriptionUsage"
                unit:
                    description:
                        - Usage unit name.
                    type: str
                    returned: always
                    sample: "Minute"
                throttle_status:
                    description:
                        - Usage throttle status.
                    type: str
                    returned: always
                    sample: "NotThrottled"

'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.mgmt.core.tools import parse_resource_id
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    pass


class AzureRMAutomationAccountInfo(AzureRMModuleBase):
    def __init__(self):
        # define user inputs into argument
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str'
            ),
            tags=dict(
                type='list',
                elements='str'
            ),
            list_statistics=dict(
                type='bool'
            ),
            list_usages=dict(
                type='bool'
            ),
            list_keys=dict(
                type='bool'
            )
        )
        # store the results of the module operation
        self.results = dict()
        self.resource_group = None
        self.name = None
        self.tags = None
        self.list_statistics = None
        self.list_usages = None
        self.list_keys = None

        super(AzureRMAutomationAccountInfo, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=False, facts_module=True)

    def exec_module(self, **kwargs):

        is_old_facts = self.module._name == 'azure_rm_automationaccount_facts'
        if is_old_facts:
            self.module.deprecate("The 'azure_rm_automationaccount_facts' module has been renamed to 'azure_rm_automationaccount_info'", version=(2.9, ))

        for key in list(self.module_arg_spec):
            setattr(self, key, kwargs[key])

        if self.resource_group and self.name:
            accounts = [self.get()]
        elif self.resource_group:
            accounts = self.list_by_resource_group()
        else:
            accounts = self.list_all()
        self.results['automation_accounts'] = [self.to_dict(x) for x in accounts if self.has_tags(x.tags, self.tags)]
        return self.results

    def to_dict(self, account):
        if not account:
            return None
        id_dict = parse_resource_id(account.id)
        result = account.as_dict()
        result['resource_group'] = id_dict['resource_group']
        if self.list_statistics:
            result['statistics'] = self.get_statics(id_dict['resource_group'], account.name)
        if self.list_usages:
            result['usages'] = self.get_usages(id_dict['resource_group'], account.name)
        if self.list_keys:
            result['keys'] = self.list_account_keys(id_dict['resource_group'], account.name)
        return result

    def get(self):
        try:
            return self.automation_client.automation_account.get(self.resource_group, self.name)
        except ResourceNotFoundError as exc:
            self.fail('Error when getting automation account {0}: {1}'.format(self.name, exc.message))

    def list_by_resource_group(self):
        result = []
        try:
            resp = self.automation_client.automation_account.list_by_resource_group(self.resource_group)
            while True:
                result.append(resp.next())
        except StopIteration:
            pass
        except Exception as exc:
            self.fail('Error when listing automation account in resource group {0}: {1}'.format(self.resource_group, exc.message))
        return result

    def list_all(self):
        result = []
        try:
            resp = self.automation_client.automation_account.list()
            while True:
                result.append(resp.next())
        except StopIteration:
            pass
        except Exception as exc:
            self.fail('Error when listing automation account: {0}'.format(exc.message))
        return result

    def get_statics(self, resource_group, name):
        result = []
        try:
            resp = self.automation_client.statistics.list_by_automation_account(resource_group, name)
            while True:
                result.append(resp.next().as_dict())
        except StopIteration:
            pass
        except Exception as exc:
            self.fail('Error when getting statics for automation account {0}/{1}: {2}'.format(resource_group, name, exc.message))
        return result

    def get_usages(self, resource_group, name):
        result = []
        try:
            resp = self.automation_client.usages.list_by_automation_account(resource_group, name)
            while True:
                result.append(resp.next().as_dict())
        except StopIteration:
            pass
        except Exception as exc:
            self.fail('Error when getting usage for automation account {0}/{1}: {2}'.format(resource_group, name, exc.message))
        return result

    def list_account_keys(self, resource_group, name):
        try:
            resp = self.automation_client.keys.list_by_automation_account(resource_group, name)
            return [x.as_dict() for x in resp.keys]
        except Exception as exc:
            self.fail('Error when listing keys for automation account {0}/{1}: {2}'.format(resource_group, name, exc.message))


def main():
    AzureRMAutomationAccountInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for iosxr_bgp_global
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: iosxr_bgp_global
short_description: Resource module to configure BGP.
description:
- This module configures and manages the attributes of BGP global on Cisco IOS-XR platforms.
version_added: 2.0.0
author: Ashwini Mhatre (@amhatre)
notes:
- This module works with connection C(network_cli).
options:
    config:
      description: A list of configurations for BGP global.
      type: dict
      suboptions:
        as_number:
          description: Autonomous system number of the router.
          type: str
        bfd: &bfd
          description: Configure BFD parameters.
          type: dict
          suboptions:
            minimum_interval: &min_interval
              description: Specifies the BFD session's minimum-interval value for the neighbor.
              type: int
            multiplier: &multiplier
              description: Specifies the BFD session's multiplier value for the neighbor.
              type: int
        bgp: &bgp
          description: BGP parameters.
          type: dict
          suboptions:
            as_path_loopcheck:
              description: Enable AS-path loop checking for iBGP peers.
              type: bool
            auto_policy_soft_reset: &auto_policy_soft_reset
              description: Enable automatic soft peer reset on policy reconfiguration.
              type: dict
              suboptions:
                disable:
                  description: Disable an automatic soft reset of Border Gateway Protocol (BGP) peers.
                  type: bool
            bestpath: &bestpath
              description: Select the bestpath selection algorithim for BGP routes.
              type: dict
              suboptions:
                as_path:
                  description: Select the bestpath selection based on as-path.
                  type: dict
                  suboptions:
                    ignore:
                      description: ignore
                      type: bool
                    multipath_relax:
                      description: multipath-relax
                      type: bool
                aigp:
                  description: AIGP attribute
                  type: dict
                  suboptions:
                    ignore:
                      description: Ignore AIGP attribute.
                      type: bool
                med:
                  description: MED attribute
                  type: dict
                  suboptions:
                    always:
                      description: Allow comparing MED from different neighbors.
                      type: bool
                    confed:
                      description: Compare MED among confederation paths.
                      type: bool
                    missing_as_worst:
                      description: Treat missing MED as the least preferred one.
                      type: bool
                compare_routerid:
                  description: Compare router-id for identical EBGP paths.
                  type: bool
                cost_community:
                  description: Cost community.
                  type: dict
                  suboptions:
                    ignore:
                      description: ignore cost_community
                      type: bool
                origin_as:
                  description: BGP origin-AS knobs.
                  type: dict
                  suboptions:
                    use:
                      description: BGP origin-AS knobs.
                      type: dict
                      suboptions:
                        validity:
                          description: BGP bestpath selection will use origin-AS validity
                          type: bool
                    allow:
                      description: BGP origin-AS knobs.
                      type: dict
                      suboptions:
                        invalid:
                          description: BGP bestpath selection will allow 'invalid' origin-AS
                          type: bool
            cluster_id: &cluster_id
              description: Cluster ID of this router acting as a route reflector.
              type: str
            confederation: &confederation
              description: confederation.
              type: dict
              suboptions:
                identifier:
                  description: Set routing domain confederation AS.
                  type: int
                peers:
                  description: Enter peer ASs in BGP confederation mode.
                  type: list
                  elements: int
            default: &default
              description: Configure default value.
              type: dict
              suboptions:
                local_preference:
                  description:
                    - local preferance.
                    - Please refer vendor documentation for valid values
                  type: int
            enforce_first_as: &enforce_first_as
              description: Enforce the first AS for EBGP routes
              type: dict
              suboptions:
                disable:
                  description: disable enforce 1st as
                  type: bool
            fast_external_fallover: &fast_external_fallover
              description: Immediately reset session if a link to a directly connected external peer goes down.
              type: dict
              suboptions:
                disable:
                  description: disable fast external fallover.
                  type: bool
            graceful_restart:
              description: Enable graceful restart support.
              type: dict
              suboptions:
                set:
                  description: Enable graceful-restart.
                  type: bool
                graceful_reset:
                  description: Reset gracefully if configuration change forces a peer reset.
                  type: bool
                restart_time: &restart_time
                  description: Restart time advertised to neighbors in seconds <1-4095>.
                  type: int
                purge_time:
                  description: Time before stale routes are purged in seconds <1-6000>.
                  type: int
                stalepath_time: &stalepath_time
                  description: Maximum time to wait for restart of GR capable peers in seconds <1-4095>.
                  type: int
            install:
              description: Install diversion path to RIB/CEF.
              type: dict
              suboptions:
                diversion:
                  description: Install diversion path to RIB/CEF.
                  type: bool
            log: &bgp_log
              description: Log bgp info
              type: dict
              suboptions:
                log_message:
                  description: Log neighbor inbound/outbound message.
                  type: dict
                  suboptions:
                    disable:
                      description: disable inbound outbound messages.
                      type: bool
                neighbor:
                  description: Log neighbor state info.
                  type: dict
                  suboptions:
                    changes:
                      description: Log neighbor up/down and reset reason.
                      type: dict
                      suboptions:
                        detail:
                          type: bool
                          description: detail
                        disable:
                          type: bool
                          description: disable
            maximum:
              description: Maximum number of neighbors that can be configured
              type: dict
              suboptions:
                neighbor:
                  description: Maximum number of neighbors <1-15000>.
                  type: int
            multipath: &multipath
              description: Change multipath selection criteria
              type: dict
              suboptions:
                as_path:
                  description: AS path
                  type: dict
                  suboptions:
                    ignore:
                      description: Ignore as-path related check for multipath selection.
                      type: dict
                      suboptions:
                        onwards:
                          description: Ignore everything onwards as-path for multipath selection.
                          type: bool
            origin_as:
              description: BGP origin-AS knobs.
              type: dict
              suboptions:
                validation:
                  description: BGP origin-AS validation knobs.
                  type: dict
                  suboptions:
                    disable:
                      description: Disable RPKI origin-AS validation.
                      type: bool
                    signal:
                      description: Signal origin-AS validity towards peers.
                      type: dict
                      suboptions:
                        ibgp:
                          description: Signal origin-AS validity towards iBGP peers
                          type: bool
                    time:
                      description: Time to wait between an RPKI update and a BGP table walk.
                      type: dict
                      suboptions:
                        time_off:
                          description: No automatic prefix validation after an RPKI update.
                          type: bool
                        time_in_second:
                          description: Prefix validation time (in seconds).
                          type: int
            redistribute_internal: &redistribute_internal
              description: Redistribute internal BGP routes.
              type: bool
            router_id: &router_id
              description: Configure Router-id. Example- A.B.C.D  IPv4 address.
              type: str
            scan_time:
              description: Configure background scanner interval for generic scanner Example- <5-3600>.
              type: int
            unsafe_ebgp_policy: &ebgp_policy
              description: Make eBGP neighbors with no policy pass all routes(cisco-support).
              type: bool
            update_delay:
              description: Set the max initial delay for sending updates Example-<0-3600> in secs.
              type: int
        default_information: &default_info
          description: Control distribution of default information.
          type: dict
          suboptions:
            originate:
              description: Distribute a default route
              type: bool
        default_metric: &default_metric
          description: Default metric. Example-<1-4294967295>.
          type: int
        graceful_maintenance:
          description: This allows the router to be brought in or out of service gracefully.
          type: dict
          suboptions:
            activate:
              description: All neighbors with graceful-maintenance config
              type: str
              choices: [ 'all-neighbors','retain-routes','all-neighbors retain-routes', '' ]
        ibgp:
          description: Set options for iBGP peers.
          type: dict
          suboptions:
            policy:
              description: Set options for route-policy.
              type: dict
              suboptions:
                out:
                  description: Set options for outbound policy.
                  type: dict
                  suboptions:
                    enforce_modifications:
                      description: Allow policy to modify all attributes.
                      type: bool
        mpls: &mpls
          description: Enable mpls parameters.
          type: dict
          suboptions:
            activate:
              description: Enter mpls interfaces in BGP mpls activate mode.
              type: dict
              suboptions:
                interface:
                  description: Name of interface to enable mpls.
                  type: str
        mvpn:
          description: Connect to PIM/PIM6.
          type: bool
        neighbors: &neighbors
          description: Specify a neighbor router.
          type: list
          elements: dict
          suboptions:
            neighbor_address:
              description:
                - Neighbor router address.
              type: str
              aliases:
                - neighbor
              required: true
            advertisement_interval:
              description: Minimum interval between sending BGP routing updates.Example-<0-600>.
              type: int
            bfd:
              description: Configure BFD parameters.
              type: dict
              suboptions:
                fast_detect:
                  description: Enable Fast detection
                  type: dict
                  suboptions:
                    set:
                      description: set fast-detect
                      type: bool
                    disable:
                      description: Prevent bfd settings from being inherited from the parent.
                      type: bool
                    strict_mode:
                      description: Hold down neighbor session until BFD session is up
                      type: bool
                multiplier: *multiplier
                minimum_interval: *min_interval
            bmp_activate: &bmp_activate
              description: Enable BMP logging for this neighbor.
              type: dict
              suboptions:
                server:
                  description: Enable BMP connection to particular server.Example-<1-8>.
                  type: int
            capability: &capability
              description: Advertise capability to the peer.
              type: dict
              suboptions:
                additional_paths:
                  description: BGP additional-paths commands.
                  type: dict
                  suboptions:
                    send:
                      type: dict
                      description: Additional paths Send capability
                      suboptions:
                        set:
                          type: bool
                          description: set send capability
                        disable:
                          type: bool
                          description: set send capability
                    receive:
                      type: dict
                      description: Additional paths receive capability
                      suboptions:
                        set:
                          type: bool
                          description: set receive capability
                        disable:
                          type: bool
                          description: set receive capability
                suppress:
                  description: Suppress advertising capability to the peer.
                  type: dict
                  suboptions:
                    four_byte_AS:
                      description: 4-byte-as capability
                      type: dict
                      suboptions:
                        set:
                          description: set 4_byte_as.
                          type: bool

                    all:
                      description: all capability
                      type: dict
                      suboptions:
                        inheritance_disable:
                          description: Do not inherit this configuration from parent group.
                          type: bool
                        set:
                          description: set all.
                          type: bool
            cluster_id: *cluster_id
            description:
              description: Neighbor specific description.
              type: str
            dmz_link_bandwidth: &dmz_link_bw
              description: Propagate the DMZ link bandwidth.
              type: dict
              suboptions:
                inheritance_disable:
                  description: Do not inherit this configuration from parent group.
                  type: bool
                set:
                  description: set dmz-link-bandwidth.
                  type: bool
            dscp:
              description: Set IP DSCP (DiffServ CodePoint).Please refer vendor document for valid entries.
              type: str
            ebgp_multihop:
              description: Allow EBGP neighbors not on directly connected networks.
              type: dict
              suboptions:
                value:
                  description: maximum hop count.Example-<1-255>.
                  type: int
                mpls:
                  description: Disable BGP MPLS forwarding.
                  type: bool
            ebgp_recv_extcommunity_dmz:
              description: Receive extcommunity dmz link bandwidth from ebgp neighbor.
              type: dict
              suboptions:
                inheritance_disable:
                  description: Prevent ebgp-recv-community-dmz from being inherited from parent
                  type: bool
                set:
                  description: set ebgp-recv-community-dmz.
                  type: bool
            ebgp_send_extcommunity_dmz:
              description: Send extcommunity dmz link bandwidth from ebgp neighbor.
              type: dict
              suboptions:
                inheritance_disable:
                  description: Prevent ebgp-send-community-dmz from being inherited from parent
                  type: bool
                cumulatie:
                  description: Send cumulative community dmz link bandwidth of all multipaths to ebgp neighbor.
                  type: bool
                set:
                  description: set ebgp-send-community-dmz.
                  type: bool
            egress_engineering:
              type: dict
              description: Enable egress peer engineering for this neighbor.
              suboptions:
                inheritance_disable:
                  description: Prevent egress-engineering from being inherited from parent
                  type: bool
                set:
                  description: set egress-engineering.
                  type: bool
            enforce_first_as: *enforce_first_as
            graceful_maintenance:
              description:
                Attributes for Graceful Maintenance. This will cause neighbors to de-prefer routes from this router and
                choose alternates. This allows the router to be brought in or out of service gracefully.
              type: dict
              suboptions:
                set:
                  description: set graceful maintenance.
                  type: bool
                activate:
                  description: Routes will be announced with the graceful maintenance attributes while activated either here or under router
                    bgp configuration.
                  type: dict
                  suboptions:
                    inheritance_disable:
                      description: Prevent activate from being inherited from the parent.
                      type: bool
                    set:
                      description: activate.
                      type: bool
                as_prepends:
                  description: Number of times to prepend the local AS number to the
                    AS path of routes. Default=0
                  type: dict
                  suboptions:
                    inheritance_disable:
                      description: Prevent as prepends from being inherited from the parent.
                      type: bool
                    value:
                      description: Range of values for as prepends.Example-<0-6> .
                      type: int
                local_preference:
                  description: local preference with which to advertise routes to ibgp neigbors. Default=No Touch
                  type: dict
                  suboptions:
                    value:
                      description: Range of values for Local Preference.Example-<0-4294967295> .
                      type: int
                    inheritance_disable:
                      description: Prevent local preference from being inherited from the parent.
                      type: bool
            graceful_restart:
              description: Enable graceful restart support for this neighbor.
              type: dict
              suboptions:
                restart_time: *restart_time
                stalepath_time: *stalepath_time
            ignore_connected_check:
              description: Bypass the directly connected nexthop check for single-hop eBGP peering
              type: dict
              suboptions:
                inheritance_disable:
                  description: Prevent ignore-connected-check from being inherited from the parent
                  type: bool
                set:
                  description: set ignore-connected-check.
                  type: bool
            keychain:
              description: Set keychain based authentication.
              type: dict
              suboptions:
                name:
                  description: Name of the key chain - maximum 32 characters.
                  type: str
                inheritance_disable:
                  description: Prevent keychain from being inherited from parent.
                  type: bool
            local:
              type: dict
              description: Configure local parameter
              suboptions:
                address:
                  description: IPv4 address
                  type: dict
                  suboptions:
                    ipv4_address:
                      description: IPv4 address <A.B.C.D>.
                      type: str
                    inheritance_disable:
                      description: Prevent local address from being inherited from parent.
                      type: bool
            local_as:
              description: Specify local AS number.
              type: dict
              suboptions:
                value:
                  description: 2 byte, 4 byte As number
                  type: int
                inheritance_disable:
                  description: Prevent local AS from being inherited from parent.
                  type: bool
                no_prepend:
                  description: Do not prepend local AS to announcements from this neighbor.
                  type: dict
                  suboptions:
                    set:
                      type: bool
                      description: Do not prepend local AS to announcements from this neighbor.
                    replace_as:
                      type: dict
                      description: Prepend only local AS to announcements to this neighbor.
                      suboptions:
                        set:
                          type: bool
                          description: Prepend only local AS to announcements to this neighbor.
                        dual_as:
                          type: bool
                          description: Dual-AS mode.
            log:
              description: Logging update messages per neighbor.
              type: dict
              suboptions:
                log_message:
                  description: Logging update/notification messages per neighbor.
                  type: dict
                  suboptions:
                    in:
                      description: Inbound log messages
                      type: dict
                      suboptions:
                        value:
                          description: Range for message log buffer size <1-100>.
                          type: int
                        disable:
                          description: Disable inbound message logging.
                          type: bool
                        inheritance_disable:
                          description: Prevents the msg log from being inherited from the parent.
                          type: bool
                    out:
                      description: Outbound log messages
                      type: dict
                      suboptions:
                        value:
                          description: Range for message log buffer size <1-100>.
                          type: int
                        disable:
                          description: Disable inbound message logging.
                          type: bool
                        inheritance_disable:
                          description: Prevents the msg log from being inherited from the parent.
                          type: bool
            origin_as:
              description: BGP origin-AS knobs.
              type: dict
              suboptions:
                validation:
                  description: BGP origin-AS validation knobs.
                  type: dict
                  suboptions:
                    disable:
                      description: Disable RPKI origin-AS validation.
                      type: bool
            password:
              type: dict
              description: Set a password.
              suboptions:
                encrypted:
                  type: str
                  description: Specifies an ENCRYPTED password will follow.
                inheritance_disable:
                  description: Prevent password from being inherited from parent.
                  type: bool
            receive_buffer_size:
              description: Set socket and BGP receive buffer size.Example <512-131072>.
              type: int
            remote_as:
              description: Neighbor Autonomous System.
              type: int
            send_buffer_size:
              description: Set socket and BGP send buffer size.Example  <4096-131072>.
              type: int
            session_open_mode:
              description: Establish BGP session using this TCP open mode.
              type: str
              choices: [ 'active-only', 'both', 'passive-only' ]
            shutdown:
              description: Administratively shut down this neighbor.
              type: dict
              suboptions:
                inheritance_disable:
                  description: Prevent shutdown from being inherited from parent
                  type: bool
                set:
                  description: shutdown.
                  type: bool
            tcp:
              description: TCP session configuration commands.
              type: dict
              suboptions:
                mss:
                  description: Maximum Segment Size.
                  type: dict
                  suboptions:
                    value:
                      description: TCP initial maximum segment size.
                      type: int
                    inheritance_disable:
                      description: Prevent mss from being inherited from parent
                      type: bool
            timers: &timers
              description: BGP per neighbor timers.
              type: dict
              suboptions:
                keepalive_time:
                  description: keepalive interval <0-65535>.
                  type: int
                holdtime:
                  description: hold time <3-65535> or 0 Disable hold time.
                  type: int
            ttl_security:
              description: Enable EBGP TTL security.
              type: dict
              suboptions:
                inheritance_disable:
                  description: Prevent ttl-security from being inherited from parent
                  type: bool
                set:
                  description: set ttl-security
                  type: bool
            update:
              description: BGP Update configuration.
              type: dict
              suboptions:
                in:
                  description: Inbound update message handling.
                  type: dict
                  suboptions:
                    filtering:
                      description: Inbound update message filtering
                      type: dict
                      suboptions:
                        attribute_filter:
                          description: Attribute-filter configuration.
                          type: dict
                          suboptions:
                            group:
                              description: Name of group.
                              type: str
                        logging:
                            description: Update filtering syslog message.
                            type: dict
                            suboptions:
                              disable:
                                description: Disable update filtering syslog message.
                                type: bool
                        update_message:
                          description: Filtered update messages.
                          type: dict
                          suboptions:
                            buffers:
                              description: Number of buffers to store filtered update messages.
                              type: int
            update_source:
              description: Source of routing updates.Refer vendor document for valid values.
              type: str
            use:
              description: Use a neighbor-group and session-group template.
              type: dict
              suboptions:
                neighbor_group:
                  description: Inherit configuration from a neighbor-group.
                  type: str
                session_group:
                  description: Inherit address-family independent config from a session-group
                  type: str
        nsr:
          description: Enable non-stop-routing support for all neighbors.
          type: dict
          suboptions:
            set:
              type: bool
              description: set nsr
            disable:
              type: bool
              description: disable nsr
        socket: &socket
          description: set socket parameters.
          type: dict
          suboptions:
            receive_buffer_size:
              description: socket receive buffer size.Example-<512-131072>.
              type: int
            send_buffer_size:
              description: socket send buffer size.Example- <4096-131072>.
              type: int
        timers: *timers
        update:
          description: BGP Update configuration.
          type: dict
          suboptions:
            in:
              description: Inbound update message handling
              type: dict
              suboptions:
                error_handling:
                  description: Inbound update message error handling.
                  type: dict
                  suboptions:
                    basic:
                      description: Inbound update message basic error handling
                      type: dict
                      suboptions:
                        ebgp:
                          type: dict
                          description: Inbound update message basic error handling for EBGP neighbors
                          suboptions:
                            disable:
                              description: disable
                              type: bool
                        ibgp:
                          type: dict
                          description: Inbound update message basic error handling for ibgp neighbors
                          suboptions:
                            disable:
                              description: disable
                              type: bool
                    extended:
                      description: Inbound update message extended error handling
                      type: dict
                      suboptions:
                        ebgp:
                          type: bool
                          description: Inbound update message extended error handling for EBGP neighbors

                        ibgp:
                          type: bool
                          description: Inbound update message extended error handling for ibgp neighbors
            out:
              description: BGP Update generation configuration.
              type: dict
              suboptions:
                logging:
                  description: Enable logging of update generation events.
                  type: bool
            limit:
              description: Upper bound on transient memory usage for update generation.Example-<16-2048>.
              type: int
        rpki:
          description: Configure RPKI.
          type: dict
          suboptions:
            route:
              description: Configure an RPKI route.A.B.C.D/length or X:X::X/length  Network/Minimum prefix length
              type: dict
              suboptions:
                value:
                  description: A.B.C.D/length or X:X::X/length  Network/Minimum prefix length.
                  type: str
                max:
                  description: Maximum prefix length. Example- <1-128>  .
                  type: int
                origin:
                  description: Origin Autonomous System number (in asplain format) Example-<1-4294967295>.
                  type: int
            servers:
              description: Configure RPKI cache-servers.
              type: list
              elements: dict
              suboptions:
                name:
                  description: address of rpki server.
                  type: str
                purge_time:
                  type: int
                  description: Time to wait after a cache goes down to clean up stale routes
                refresh_time:
                  type: dict
                  description: Time between sending serial-queries for the RPKI cache-server
                  suboptions:
                    value:
                      description: Purge time (in seconds) <30-360>
                      type: int
                    time_off:
                      description: Do not send serial-queries periodically
                      type: bool
                response_time:
                  type: dict
                  description: Time to wait for a response from the RPKI cache-server
                  suboptions:
                    value:
                      description: Purge time (in seconds) <15-3600>
                      type: int
                    time_off:
                      description: Wait indefinitely for a response
                      type: bool
                shutdown:
                  type: bool
                  description: Shutdown the RPKI cache-server
                transport:
                  type: dict
                  description: Specify a transport method for the RPKI cache-server
                  suboptions:
                    ssh:
                      description: Connect to the RPKI cache-server using SSH
                      type: dict
                      suboptions:
                        port:
                          description: Specify a port number for the RPKI cache-server transport
                          type: int
                    tcp:
                      description: Connect to the RPKI cache-server using TCP (unencrypted)
                      type: dict
                      suboptions:
                        port:
                          description: Specify a port number for the RPKI cache-server transport
                          type: int
        vrfs:
          description: Specify a vrf name.
          type: list
          elements: dict
          suboptions:
            vrf:
              description: VRF name.
              type: str
            bfd: *bfd
            bgp:
              description: BGP commands.
              type: dict
              suboptions:
                auto_policy_soft_reset: *auto_policy_soft_reset
                bestpath: *bestpath
                default: *default
                enforce_first_as: *enforce_first_as
                fast_external_fallover: *fast_external_fallover
                log: *bgp_log
                multipath: *multipath
                redistribute_internal: *redistribute_internal
                router_id: *router_id
                unsafe_ebgp_policy: *ebgp_policy
            default_information: *default_info
            default_metric: *default_metric
            mpls: *mpls
            neighbors: *neighbors
            rd:
              description: route distinguisher.
              type: dict
              suboptions:
                auto:
                  description: Automatic route distinguisher.
                  type: bool
            socket: *socket
            timers: *timers
    running_config:
      description:
        The state the configuration should be left in.
        - State I(purged) removes all the BGP configurations from the
        target device. Use caution with this state.
        - State I(deleted) only removes BGP attributes that this modules
        manages and does not negate the BGP process completely. Thereby, preserving
        address-family related configurations under BGP context.
        - Running states I(deleted) and I(replaced) will result in an error if there
        are address-family configuration lines present under a neighbor,
        or a vrf context that is to be removed. Please use the
        M(cisco.iosxr.iosxr_bgp_address_family) or M(cisco.iosxr.iosxr_bgp_neighbor_address_family)
        modules for prior cleanup.
        - Refer to examples for more details.
      type: str
    state:
      description:
      - The state the configuration should be left in.
      type: str
      choices: [deleted, merged, replaced, gathered, rendered, parsed, purged, overridden]
      default: merged
"""
EXAMPLES = """

# Using merged
#
# Before state
# ------------
# RP/0/0/CPU0:10#show running-config router bgp
# Thu Feb  4 09:38:36.245 UTC
# % No such configuration item(s)
# RP/0/0/CPU0:10#

- name: Merge the following BGP global configuration
  cisco.iosxr.iosxr_bgp_global:
    config:
      as_number: 65536
      default_metric: 5
      socket:
        receive_buffer_size: 514
        send_buffer_size: 4098
      bgp:
        confederation:
          identifier: 4
        bestpath:
          med:
            confed: true
        cluster_id: 5
        router_id: 192.0.2.10
      neighbors:
        - neighbor: 192.0.2.13
          remote_as: 65538
          bfd:
            fast_detect:
              strict_mode: true
            multiplier: 6
            minimum_interval: 20
      vrfs:
        - vrf: vrf1
          default_metric: 5

#
# Task Output:
# ---------------
#
# before: {}
# commands:
#   - router bgp 65536
#   - bgp cluster-id 5
#   - bgp router-id 192.0.2.10
#   - bgp bestpath med confed
#   - bgp confederation identifier 4
#   - default-metric 5
#   - socket receive-buffer-size 514
#   - socket send-buffer-size 4098
#   - neighbor 192.0.2.13
#   - bfd fast-detect strict-mode
#   - bfd minimum-interval 20
#   - bfd multiplier 6
#   - remote-as 65538
#   - vrf vrf1
#   - default-metric 5
#
# after:
#     as_number: '65536'
#     bgp:
#       bestpath:
#         med:
#           confed: true
#       cluster_id: '5'
#       confederation:
#         identifier: 4
#       router_id: 192.0.2.10
#     default_metric: 5
#     neighbors:
#     - bfd:
#         fast_detect:
#           strict_mode: true
#         minimum_interval: 20
#         multiplier: 6
#       neighbor_address: 192.0.2.13
#       remote_as: 65538
#     socket:
#       receive_buffer_size: 514
#       send_buffer_size: 4098
#     vrfs:
#     - default_metric: 5
#       vrf: vrf1
#
# After state
# -----------

# RP/0/0/CPU0:10#show running-config router bgp
# Thu Feb  4 09:44:32.480 UTC
# router bgp 65536
#  bgp confederation identifier 4
#  bgp router-id 192.0.2.10
#  bgp cluster-id 5
#  default-metric 5
#  socket send-buffer-size 4098
#  bgp bestpath med confed
#  socket receive-buffer-size 514
#  neighbor 192.0.2.13
#   remote-as 65538
#   bfd fast-detect strict-mode
#   bfd multiplier 6
#   bfd minimum-interval 20
#  !
#  vrf vrf1
#   default-metric 5
#  !
# !

# Using replaced
#
# Before state
# ------------
#
# RP/0/0/CPU0:10#show running-config router bgp
# Thu Feb  4 09:44:32.480 UTC
# router bgp 65536
#  bgp confederation identifier 4
#  bgp router-id 192.0.2.10
#  bgp cluster-id 5
#  default-metric 5
#  socket send-buffer-size 4098
#  bgp bestpath med confed
#  socket receive-buffer-size 514
#  neighbor 192.0.2.13
#   remote-as 65538
#   bfd fast-detect strict-mode
#   bfd multiplier 6
#   bfd minimum-interval 20
#  !
#  vrf vrf1
#   default-metric 5
#  !
# !

- name: Replace the following configuration
  cisco.iosxr.iosxr_bgp_global:
    state: replaced
    config:
      as_number: 65536
      default_metric: 4
      socket:
        receive_buffer_size: 514
        send_buffer_size: 4098
      bgp:
        confederation:
          identifier: 4
        bestpath:
          med:
            confed: true
        cluster_id: 5
        router_id: 192.0.2.10
      neighbors:
        - neighbor: 192.0.2.14
          remote_as: 65538
          bfd:
            fast_detect:
              strict_mode: true
            multiplier: 6
            minimum_interval: 20
      vrfs:
        - vrf: vrf1
          default_metric: 5

#
# Task Output:
# -------------
#
# before:
#     as_number: '65536'
#     bgp:
#       bestpath:
#         med:
#           confed: true
#       cluster_id: '5'
#       confederation:
#         identifier: 4
#       router_id: 192.0.2.10
#     default_metric: 5
#     neighbors:
#     - bfd:
#         fast_detect:
#           strict_mode: true
#         minimum_interval: 20
#         multiplier: 6
#       neighbor_address: 192.0.2.13
#       remote_as: 65538
#     socket:
#       receive_buffer_size: 514
#       send_buffer_size: 4098
#     vrfs:
#     - default_metric: 5
#       vrf: vrf1
#
# commands:
#   - router bgp 65536
#   - default-metric 4
#   - neighbor 192.0.2.14
#   - bfd fast-detect strict-mode
#   - bfd minimum-interval 20
#   - bfd multiplier 6
#   - remote-as 65538
#   - no neighbor 192.0.2.13
#
# after:
#     as_number: '65536'
#     bgp:
#       bestpath:
#         med:
#           confed: true
#       cluster_id: '5'
#       confederation:
#         identifier: 4
#       router_id: 192.0.2.10
#     default_metric: 4
#     neighbors:
#     - bfd:
#         fast_detect:
#           strict_mode: true
#         minimum_interval: 20
#         multiplier: 6
#       neighbor_address: 192.0.2.14
#       remote_as: 65538
#     socket:
#       receive_buffer_size: 514
#       send_buffer_size: 4098
#     vrfs:
#     - default_metric: 5
#       vrf: vrf1
#
# After state
# -----------
#
# RP/0/0/CPU0:10#show running-config router bgp
# Thu Feb  4 09:54:11.161 UTC
# router bgp 65536
#  bgp confederation identifier 4
#  bgp router-id 192.0.2.10
#  bgp cluster-id 5
#  default-metric 4
#  socket send-buffer-size 4098
#  bgp bestpath med confed
#  socket receive-buffer-size 514
#  neighbor 192.0.2.14
#   remote-as 65538
#   bfd fast-detect strict-mode
#   bfd multiplier 6
#   bfd minimum-interval 20
#  !
#  vrf vrf1
#   default-metric 5
#  !
# !

# Using overridden
#
# Before state
# ------------
#
# RP/0/0/CPU0:10#show running-config router bgp
# Thu Feb  4 09:44:32.480 UTC
# router bgp 65536
#  bgp confederation identifier 4
#  bgp router-id 192.0.2.10
#  bgp cluster-id 5
#  default-metric 5
#  socket send-buffer-size 4098
#  bgp bestpath med confed
#  socket receive-buffer-size 514
#  neighbor 192.0.2.13
#   remote-as 65538
#   bfd fast-detect strict-mode
#   bfd multiplier 6
#   bfd minimum-interval 20
#  !
#  vrf vrf1
#   default-metric 5
#  !
# !

- name: Override running config with provided configuration
  cisco.iosxr.iosxr_bgp_global:
    state: overridden
    config:
      as_number: 65536
      default_metric: 4
      socket:
        receive_buffer_size: 514
        send_buffer_size: 4098
      bgp:
        confederation:
          identifier: 4
        bestpath:
          med:
            confed: true
        cluster_id: 5
        router_id: 192.0.2.10
      neighbors:
        - neighbor: 192.0.2.14
          remote_as: 65538
          bfd:
            fast_detect:
              strict_mode: true
            multiplier: 6
            minimum_interval: 20
      vrfs:
        - vrf: vrf1
          default_metric: 5
#
# Task Output:
# -------------
#
# before:
#     as_number: '65536'
#     bgp:
#       bestpath:
#         med:
#           confed: true
#       cluster_id: '5'
#       confederation:
#         identifier: 4
#       router_id: 192.0.2.10
#     default_metric: 5
#     neighbors:
#     - bfd:
#         fast_detect:
#           strict_mode: true
#         minimum_interval: 20
#         multiplier: 6
#       neighbor_address: 192.0.2.13
#       remote_as: 65538
#     socket:
#       receive_buffer_size: 514
#       send_buffer_size: 4098
#     vrfs:
#     - default_metric: 5
#       vrf: vrf1
#
# commands:
#   - router bgp 65536
#   - default-metric 4
#   - neighbor 192.0.2.14
#   - bfd fast-detect strict-mode
#   - bfd minimum-interval 20
#   - bfd multiplier 6
#   - remote-as 65538
#   - no neighbor 192.0.2.13
#
# after:
#     as_number: '65536'
#     bgp:
#       bestpath:
#         med:
#           confed: true
#       cluster_id: '5'
#       confederation:
#         identifier: 4
#       router_id: 192.0.2.10
#     default_metric: 4
#     neighbors:
#     - bfd:
#         fast_detect:
#           strict_mode: true
#         minimum_interval: 20
#         multiplier: 6
#       neighbor_address: 192.0.2.14
#       remote_as: 65538
#     socket:
#       receive_buffer_size: 514
#       send_buffer_size: 4098
#     vrfs:
#     - default_metric: 5
#       vrf: vrf1

# After state
# -----------
#
# RP/0/0/CPU0:10#show running-config router bgp
# Thu Feb  4 09:54:11.161 UTC
# router bgp 65536
#  bgp confederation identifier 4
#  bgp router-id 192.0.2.10
#  bgp cluster-id 5
#  default-metric 4
#  socket send-buffer-size 4098
#  bgp bestpath med confed
#  socket receive-buffer-size 514
#  neighbor 192.0.2.14
#   remote-as 65538
#   bfd fast-detect strict-mode
#   bfd multiplier 6
#   bfd minimum-interval 20
#  !
#  vrf vrf1
#   default-metric 5
#  !
# !

# Using deleted
#
# Before state
# ------------
#
# RP/0/0/CPU0:10#show running-config router bgp
# Thu Feb  4 09:54:11.161 UTC
# router bgp 65536
#  bgp confederation identifier 4
#  bgp router-id 192.0.2.10
#  bgp cluster-id 5
#  default-metric 4
#  socket send-buffer-size 4098
#  bgp bestpath med confed
#  socket receive-buffer-size 514
#  neighbor 192.0.2.14
#   remote-as 65538
#   bfd fast-detect strict-mode
#   bfd multiplier 6
#   bfd minimum-interval 20
#  !
#  vrf vrf1
#   default-metric 5
#  !
# !
#

- name: Delete BGP configurations handled by this module
  cisco.iosxr.iosxr_bgp_global:
    config:
      as_number: 65536
    state: deleted

#
# Task Output:
# -------------
#
# before:
#     as_number: '65536'
#     bgp:
#       bestpath:
#         med:
#           confed: true
#       cluster_id: '5'
#       confederation:
#         identifier: 4
#       router_id: 192.0.2.10
#     default_metric: 4
#     neighbors:
#     - bfd:
#         fast_detect:
#           strict_mode: true
#         minimum_interval: 20
#         multiplier: 6
#       neighbor_address: 192.0.2.14
#       remote_as: 65538
#     socket:
#       receive_buffer_size: 514
#       send_buffer_size: 4098
#     vrfs:
#     - default_metric: 5
#       vrf: vrf1
#
# commands:
#   - router bgp 65536
#   - no bgp cluster-id 5
#   - no bgp router-id 192.0.2.10
#   - no bgp bestpath med confed
#   - no bgp confederation identifier 4
#   - no default-metric 4
#   - no socket receive-buffer-size 514
#   - no socket send-buffer-size 4098
#   - no neighbor 192.0.2.14
#   - no vrf vrf1
#
# after:
#     as_number: '65536'
#
# After state
# -----------
#
# RP/0/0/CPU0:10#show running-config router bgp
# Thu Feb  4 10:01:08.232 UTC
# router bgp 65536
# !
#

# Using purged
#
# Before state
# ------------
#
# RP/0/0/CPU0:10#show running-config router bgp
# Thu Feb  4 09:54:11.161 UTC
# router bgp 65536
#  bgp confederation identifier 4
#  bgp router-id 192.0.2.10
#  bgp cluster-id 5
#  default-metric 5
#  socket send-buffer-size 4098
#  bgp bestpath med confed
#  socket receive-buffer-size 514
#  neighbor 192.0.2.13
#   remote-as 65538
#   bfd fast-detect strict-mode
#   bfd multiplier 6
#   bfd minimum-interval 20
#  !
#  vrf vrf1
#   default-metric 5
#  !
# !
#

- name: Purge all BGP configurations from the device
  cisco.iosxr.iosxr_bgp_global:
    state: purged

#
# Task Output:
# -------------
#
# before:
#     as_number: '65536'
#     bgp:
#       bestpath:
#         med:
#           confed: true
#       cluster_id: '5'
#       confederation:
#         identifier: 4
#       router_id: 192.0.2.10
#     default_metric: 5
#     neighbors:
#     - bfd:
#         fast_detect:
#           strict_mode: true
#         minimum_interval: 20
#         multiplier: 6
#       neighbor_address: 192.0.2.13
#       remote_as: 65538
#     socket:
#       receive_buffer_size: 514
#       send_buffer_size: 4098
#     vrfs:
#     - default_metric: 5
#       vrf: vrf1
#
# commands:
#   - no router bgp 65536
#
# after: {}
#
# After state
# -----------
#
# RP/0/0/CPU0:10#show running-config router bgp
# Thu Feb  4 09:38:36.245 UTC
# % No such configuration item(s)
# RP/0/0/CPU0:10#
#

#
# Using Rendered
# -----------------
#
- name: >-
    Render platform specific configuration lines (without connecting to the
    device)
  cisco.iosxr.iosxr_bgp_global:
    state: rendered
    config:
      as_number: 1
      default_metric: 4
      vrfs:
        - vrf: vrf3
          bfd:
            minimum_interval: 20
            multiplier: 10
          bgp:
            fast_external_fallover:
              disable: true
            router_id: 1.2.3.4
            auto_policy_soft_reset:
              disable: true
          timers:
            keepalive_time: 20
            holdtime: 30
        - vrf: vrf2
          bgp:
            enforce_first_as:
              disable: true
          default_metric: 4
          neighbors:
            - neighbor: 1.1.1.3
              remote_as: 2
              graceful_maintenance:
                set: true
                activate:
                  inheritance_disable: true
                local_preference:
                  value: 1
                as_prepends:
                  value: 2

#
# Task output
# -----------------------
# rendered:
#   - router bgp 1
#   - default-metric 4
#   - vrf vrf3
#   - bfd multiplier 10
#   - bfd minimum-interval 20
#   - bgp auto-policy-soft-reset disable
#   - bgp fast-external-fallover disable
#   - bgp router-id 1.2.3.4
#   - timers bgp 20 30
#   - vrf vrf2
#   - neighbor 1.1.1.3
#   - remote-as 2
#   - graceful-maintenance
#   - graceful-maintenance activate inheritance-disable
#   - graceful-maintenance local-preference 1
#   - graceful-maintenance as-prepends 2
#   - bgp enforce-first-as disable
#   - default-metric 4

# Using parsed
#
#  parsed.cfg
#  ------------
# router bgp 65536
#  bgp confederation identifier 4
#  bgp router-id 192.0.2.10
#  bgp cluster-id 5
#  default-metric 4
#  socket send-buffer-size 4098
#  bgp bestpath med confed
#  socket receive-buffer-size 514
#  neighbor 192.0.2.11
#   remote-as 65537
#   cluster-id 3
#  !
#  neighbor 192.0.2.14
#   remote-as 65538
#   bfd fast-detect strict-mode
#   bfd multiplier 6
#   bfd minimum-interval 20
#  !
# !
#

- name: Parse externally provided BGP config
  cisco.iosxr.iosxr_bgp_global:
    running_config: "{{ lookup('file', 'parsed.cfg') }}"
    state: parsed

# Task output
# -----------------------
# parsed:
#     as_number: '65536'
#     bgp:
#       bestpath:
#         med:
#           confed: true
#       cluster_id: '5'
#       confederation:
#         identifier: 4
#       router_id: 192.0.2.10
#     default_metric: 4
#     neighbors:
#     - cluster_id: '3'
#       neighbor_address: 192.0.2.11
#       remote_as: 65537
#     - bfd:
#         fast_detect:
#           strict_mode: true
#         minimum_interval: 20
#         multiplier: 6
#       neighbor_address: 192.0.2.14
#       remote_as: 65538
#     socket:
#       receive_buffer_size: 514
#       send_buffer_size: 4098

# Using gathered
#
# Before state
# ------------
#

# RP/0/0/CPU0:10#show running-config router bgp
# Thu Feb  4 09:38:36.245 UTC
# router bgp 65536
#  bgp confederation identifier 4
#  bgp router-id 192.0.2.10
#  bgp cluster-id 5
#  default-metric 5
#  socket send-buffer-size 4098
#  bgp bestpath med confed
#  socket receive-buffer-size 514
#  neighbor 192.0.2.13
#   remote-as 65538
#   bfd fast-detect strict-mode
#   bfd multiplier 6
#   bfd minimum-interval 20
#  !
#  vrf vrf1
#   default-metric 5
#  !
# !

- name: Gather bgp global facts
  cisco.iosxr.iosxr_bgp_global:
    state: gathered

# Task Output:
# ------------
#
# gathered:
#     as_number: '65536'
#     bgp:
#       bestpath:
#         med:
#           confed: true
#       cluster_id: '5'
#       confederation:
#         identifier: 4
#       router_id: 192.0.2.10
#     default_metric: 5
#     neighbors:
#     - bfd:
#         fast_detect:
#           strict_mode: true
#         minimum_interval: 20
#         multiplier: 6
#       neighbor_address: 192.0.2.13
#       remote_as: 65538
#     socket:
#       receive_buffer_size: 514
#       send_buffer_size: 4098
#     vrfs:
#     - default_metric: 5
#       vrf: vrf1
"""
RETURN = """
before:
  description: The configuration prior to the model invocation.
  returned: always
  type: dict
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
after:
  description: The resulting configuration model invocation.
  returned: when changed
  type: dict
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
commands:
  description: The set of commands pushed to the remote device.
  returned: always
  type: list
  sample:
  - router bgp 65536
  - bgp cluster-id 5
  - bgp router-id 192.0.2.10
  - bgp bestpath med confed

rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:
  - router bgp 1
  - default-metric 4
  - vrf vrf3
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.argspec.bgp_global.bgp_global import (
    Bgp_globalArgs,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.config.bgp_global.bgp_global import (
    Bgp_global,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Bgp_globalArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Bgp_global(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat Inc.
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The ios_acls class
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to it's desired end-state is
created
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

from ansible.module_utils._text import to_text
from ansible.module_utils.six import iteritems
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
)

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.facts.facts import Facts
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.rm_templates.acls import (
    AclsTemplate,
)


class Acls(ResourceModule):
    """
    The ios_acls config class
    """

    def __init__(self, module):
        super(Acls, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="acls",
            tmplt=AclsTemplate(),
        )

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        if self.state not in ["parsed", "gathered"]:
            self.generate_commands()
            self.run_commands()
        return self.result

    def generate_commands(self):
        """Generate configuration commands to send based on
        want, have and desired state.
        """
        haved, wantd = dict(), dict()

        if self.have:
            haved = self.list_to_dict(self.have)
        if self.want:
            wantd = self.list_to_dict(self.want)

        # if state is merged, merge want onto have and then compare
        if self.state == "merged":
            wantd = dict_merge(haved, wantd)

        # if state is deleted, empty out wantd and set haved to want
        if self.state == "deleted":
            haved = {k: v for k, v in iteritems(haved) if k in wantd or not wantd}
            if wantd.get("ipv4") and not haved.get("ipv4"):
                haved["ipv4"] = {}
            if wantd.get("ipv6") and not haved.get("ipv6"):
                haved["ipv6"] = {}
            for key, hvalue in iteritems(haved):
                wvalue = wantd.pop(key, {})
                if wvalue:
                    wplists = wvalue.get("acls", {})
                    hplists = hvalue.get("acls", {})
                    hvalue["acls"] = {
                        k: v for k, v in iteritems(hplists) if k in wplists or not wplists
                    }

        # remove superfluous config for overridden and deleted
        if self.state in ["overridden", "deleted"]:
            for k, have in iteritems(haved):
                if k not in wantd:
                    self._compare(want={}, have=have, afi=k)

        for k, want in iteritems(wantd):
            self._compare(want=want, have=haved.pop(k, {}), afi=k)

    def _compare(self, want, have, afi):
        """Leverages the base class `compare()` method and
        populates the list of commands to be run by comparing
        the `want` and `have` data with the `parsers` defined
        for the acls network resource.
        """

        def rearrange_cmds(aces):
            non_negates = []
            negates = []
            for ace in aces:
                if ace.startswith("no"):
                    negates.append(ace)
                else:
                    non_negates.append(ace)
            if non_negates or negates:
                negates.extend(non_negates)
            return negates

        wplists = want.get("acls", {})
        hplists = have.get("acls", {})
        for wname, wentry in iteritems(wplists):
            hentry = hplists.pop(wname, {})
            acl_type = wentry["acl_type"] if wentry.get("acl_type") else hentry.get("acl_type")
            # If ACLs type is different between existing and wanted ACL, we need first remove it
            if acl_type != hentry.get("acl_type", acl_type):
                self.commands.append(
                    "no " + self.acl_name_cmd(wname, afi, hentry.get("acl_type", "")),
                )
                hentry.pop(
                    "aces",
                    {},
                )  # We remove ACEs because we have previously add a command to suppress completely the ACL

            begin = len(self.commands)  # to determine the index for acl command
            self._compare_aces(
                wentry.pop("aces", {}),
                hentry.pop("aces", {}),
                afi,
                wname,
            )  # handle aces

            end = len(self.commands)
            self.commands[begin:end] = rearrange_cmds(self.commands[begin:])

            if len(self.commands) != begin or (not have and want):
                _cmd = self.acl_name_cmd(wname, afi, acl_type)
                self.commands.insert(begin, _cmd)

        if self.state in ["overridden", "deleted"]:
            # remove remaining acls lists
            for hname, hval in iteritems(hplists):
                _cmd = self.acl_name_cmd(hname, afi, hval.get("acl_type", ""))
                self.commands.append("no " + _cmd)

    def _compare_aces(self, want, have, afi, name):
        """compares all aces"""

        def add_afi(entry, afi):
            """adds afi needed for
            setval processing"""
            if entry:
                entry["afi"] = afi
            return entry

        def pop_remark(r_entry, afi):
            """Takes out remarks from ace entry as remarks not same
            does not mean the ace entry to be re-introduced
            """
            if r_entry.get("remarks"):
                return r_entry.pop("remarks")
            else:
                return {}

        # case 1 - loop on want and compare with have data here
        for wseq, wentry in iteritems(want):
            hentry = have.pop(wseq, {})
            rem_hentry, rem_wentry = {}, {}

            if hentry:  # if there is have information with same sequence
                # the protocol options are processed here
                hentry = self.sanitize_protocol_options(wentry, hentry)

            if hentry != wentry:  # if want and have is different
                if hentry:  # separate remarks from have in an ace entry
                    rem_hentry["remarks"] = pop_remark(hentry, afi)
                if wentry:  # separate remarks from want in an ace entry
                    rem_wentry["remarks"] = pop_remark(wentry, afi)

                if hentry:  # have aces processing starts here
                    if self.state == "merged":
                        self._module.fail_json(
                            msg="Cannot update existing sequence {0} of ACLs {1} with state merged."
                            " Please use state replaced or overridden.".format(
                                hentry.get("sequence", ""),
                                name,
                            ),
                        )  # if merged then don't update anything and fail

                    # i.e if not merged
                    if rem_hentry.get("remarks") != rem_wentry.get("remarks"):
                        self.addcmd(
                            {
                                "sequence": hentry.get("sequence", None),
                            },
                            "remarks_no_data",
                            negate=True,
                        )  # remove all remarks for a ace if want and have don't match
                        # as if we randomly add aces we cannot maintain order we have to
                        # add all of them again, for that ace
                        rem_hentry["remarks"] = {}
                        # and me empty our have as we would add back
                        # all our remarks for that ace anyways

                    # remove ace if not in want
                    # we might think why not update it directly,
                    # if we try to update without negating the entry appliance
                    # reports % Duplicate sequence number
                    if hentry != wentry:
                        self.addcmd(add_afi(hentry, afi), "aces", negate=True)
                        # once an ace is negated intentionally emptying out have so that
                        # the remarks are repopulated, as the remarks and ace behavior is sticky
                        # if an ace is taken out all the remarks is removed automatically.
                        rem_hentry["remarks"] = {}

                if rem_wentry.get("remarks"):  # add remark if not in have
                    if rem_hentry.get("remarks"):
                        self.addcmd(
                            {
                                "sequence": hentry.get("sequence", None),
                            },
                            "remarks_no_data",
                            negate=True,
                        )  # but delete all remarks before to protect order
                    for k_wrems, wrems in rem_wentry.get("remarks").items():
                        self.addcmd(
                            {
                                "remarks": wrems,
                                "sequence": wentry.get("sequence", ""),
                            },
                            "remarks",
                        )

                # add ace if not in have
                if hentry != wentry:
                    if len(wentry) == 1 and wentry.get(
                        "sequence",
                    ):  # if the ace entry just has sequence then do nothing
                        continue
                    else:  # add normal ace entries from want
                        self.addcmd(add_afi(wentry, afi), "aces")

        # case 2 - loop over remaining have and remove them
        for hseq in have.values():
            if hseq.get("remarks"):  # remove all remarks in that
                self.addcmd(
                    {
                        "sequence": hseq.get("sequence", None),
                    },
                    "remarks_no_data",
                    negate=True,
                )
                hseq.pop("remarks")
            # deal with the rest of ace entry
            self.addcmd(
                add_afi(hseq, afi),
                "aces",
                negate=True,
            )

    def sanitize_protocol_options(self, wace, hace):
        """handles protocol and protocol options as optional attribute"""
        if wace.get("protocol_options"):
            if not wace.get("protocol") and (
                list(wace.get("protocol_options"))[0] == hace.get("protocol")
            ):
                hace.pop("protocol")
                hace["protocol_options"] = wace.get("protocol_options")
        return hace

    def acl_name_cmd(self, name, afi, acl_type):
        """generate parent acl command"""

        if afi == "ipv4":
            if not acl_type:
                try:
                    acl_id = int(name)
                    if not acl_type:
                        if acl_id >= 1 and acl_id <= 99:
                            acl_type = "standard"
                        if acl_id >= 100 and acl_id <= 199:
                            acl_type = "extended"
                except ValueError:
                    acl_type = "extended"
            command = "ip access-list {0} {1}".format(acl_type, name)
        elif afi == "ipv6":
            command = "ipv6 access-list {0}".format(name)
        return command

    def list_to_dict(self, param):
        """converts list attributes to dict"""

        temp = dict()
        if param:
            for each in param:  # ipv4 and ipv6 acl
                temp_acls = {}
                if each.get("acls"):
                    for acl in each.get("acls"):  # check each acl for aces
                        temp_aces = {}
                        if acl.get("aces"):
                            rem_idx = 0  # remarks if defined in an ace
                            for count, ace in enumerate(
                                acl.get("aces"),
                            ):  # each ace turned to dict
                                if (
                                    ace.get("destination")
                                    and ace.get("destination", {}).get(
                                        "port_protocol",
                                        {},
                                    )
                                    and not ace.get("destination", {})
                                    .get("port_protocol", {})
                                    .get("range")
                                ):
                                    for k, v in (
                                        ace.get("destination", {}).get("port_protocol", {}).items()
                                    ):
                                        ace["destination"]["port_protocol"][k] = (
                                            self.port_protocl_no_to_protocol(v, ace.get("protocol"))
                                        )
                                if acl.get("acl_type") == "standard":
                                    for ks in list(ace.keys()):
                                        if ks not in [
                                            "sequence",
                                            "grant",
                                            "source",
                                            "remarks",
                                            "log",
                                        ]:  # failing for mutually exclusive standard acl key
                                            self._module.fail_json(
                                                "Unsupported attribute for standard ACL - {0}.".format(
                                                    ks,
                                                ),
                                            )

                                if ace.get(
                                    "remarks",
                                ):  # index aces inside of each ace don't cluster them all
                                    rem_ace = {}
                                    # en_name = str(acl.get("name")) + "remark"
                                    # temp_rem.extend(ace.pop("remarks"))
                                    for remks in ace.get("remarks"):
                                        rem_ace[remks.replace(" ", "_")] = remks
                                    ace["remarks"] = rem_ace

                                if ace.get("sequence"):
                                    temp_aces.update({ace.get("sequence"): ace})
                                elif ace.get("remarks"):
                                    temp_aces.update({"__{0}".format(rem_idx): ace})
                                    rem_idx += 1
                                elif ace:
                                    temp_aces.update({"_" + to_text(count): ace})

                            # if temp_rem:  # add remarks to the temp ace
                            #     temp_aces.update({en_name: {"remarks": temp_rem}})

                        if acl.get("acl_type"):  # update acl dict with req info
                            temp_acls.update(
                                {
                                    acl.get("name"): {
                                        "aces": temp_aces,
                                        "acl_type": acl["acl_type"],
                                    },
                                },
                            )
                        else:  # if no acl type then here eg: ipv6
                            temp_acls.update({acl.get("name"): {"aces": temp_aces}})
                each["acls"] = temp_acls
                temp.update({each["afi"]: {"acls": temp_acls}})
            return temp

    def port_protocl_no_to_protocol(self, num, protocol):
        map_protocol = {
            "179": "bgp",
            "19": "chargen",
            "514": "cmd",
            "13": "daytime",
            "9": "discard",
            "53": "domain",
            "7": "echo",
            "512": "exec",
            "79": "finger",
            "21": "ftp",
            "20": "ftp-data",
            "70": "gopher",
            "101": "hostname",
            "113": "ident",
            "194": "irc",
            "543": "klogin",
            "544": "kshell",
            "513": "login",
            "515": "lpd",
            "135": "msrpc",
            "119": "nntp",
            "5001": "onep-plain",
            "5002": "onep-tls",
            "496": "pim-auto-rp",
            "109": "pop2",
            "110": "pop3",
            "25": "smtp",
            "111": "sunrpc",
            "49": "tacacs",
            "517": "talk",
            "23": "telnet",
            "37": "time",
            "540": "uucp",
            "43": "whois",
            "80": "www",
        }  # NOTE - "514": "syslog" duplicate value device renders "cmd"
        if protocol == "udp" and num in ["135"]:
            return num
        return map_protocol.get(num, num)



#!/usr/bin/python
#
# Copyright (c) 2022 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_aksagentpool_info
version_added: '1.14.0'
short_description: Show the details for a node pool in the managed Kubernetes cluster
description:
    - Get the details for a node pool in the managed Kubernetes cluster.

options:
    resource_group:
        description:
            - The name of the resource group.
        type: str
        required: True
    cluster_name:
        description:
            - The cluster name.
        type: str
        required: True
    name:
        description:
            - The node pool name.
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - xuzhang3 (@xuzhang3)
    - Fred Sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Get node agent pool by cluster name
  azure_rm_aksagentpool_info:
    resource_group: myRG
    cluster_name: testcluster

- name: Get node agent pool by name
  azure_rm_aksagentpool_info:
    resource_group: myRG
    cluster_name: testcluster
    name: default
'''

RETURN = '''
aks_agent_pools:
    description:
        - Details for a node pool in the managed Kubernetes cluster.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID.
            type: str
            returned: always
            sample: "/subscriptions/xxx-xxxf/resourcegroups/myRG/providers/Microsoft.ContainerService/managedClusters/cluster/agentPools/default"
        resource_group:
            description:
                - Resource group name.
            type: str
            returned: always
            sample: myRG
        name:
            description:
                - Resource name.
            type: str
            returned: always
            sample: default
        cluster_name:
            description:
                - The cluster name.
            type: str
            returned: always
            sample: testcluster
        availability_zones:
            description:
                - Availability zones for nodes. Must use VirtualMachineScaleSets AgentPoolType.
            type: list
            returned: always
            sample: [1, 2]
        count:
            description:
                - Number of agents (VMs) to host docker containers.
            type: int
            returned: always
            sample: 2
        enable_auto_scaling:
            description:
                - Whether to enable auto-scaler.
            type: bool
            returned: always
            sample: null
        enable_node_public_ip:
            description:
                -  Enable public IP for nodes.
            type: bool
            returned: always
            sample: True
        max_count:
            description:
                - Maximum number of nodes for auto-scaling.
            type: int
            returned: always
            sample: 10
        min_count:
            description:
                - Minimum number of nodes for auto-scaling.
            type: int
            returned: always
            sample: 1
        max_pods:
            description:
                - Maximum number of pods that can run on a node.
            type: int
            returned: always
            sample: 42
        mode:
            description:
                - AgentPoolMode represents mode of an agent pool.
            type: str
            returned: always
            sample: System
        node_image_version:
            description:
                - Version of node image.
            type: str
            returned: always
            sample: AKSUbuntu-1804gen2containerd-2022.08.23
        node_labels:
            description:
                - Agent pool node labels to be persisted across all nodes in agent pool.
            type: list
            returned: always
            sample: ["release":"stable"]
        node_taints:
            description:
                - Taints added to new nodes during node pool create and scale.
            type: list
            returned: always
            sample: ["CriticalAddonsOnly=false:NoSchedule"]
        orchestrator_version:
            description:
                - Version of orchestrator specified when creating the managed cluster.
            type: str
            returned: always
            sample: 1.22.11
        os_disk_size_gb:
            description:
                - OS Disk Size in GB to be used to specify the disk size for every machine in this master agent pool.
            type: int
            returned: always
            sample: 128
        os_type:
            description:
                - OsType to be used to specify os type.
            type: str
            returned: always
            sample: Linux
        os_sku:
            description:
                - OS SKU to be used to specify os type.
            type: str
            returned: always
            sample: Windows2022
        provisioning_state:
            description:
                - The current deployment or provisioning state, which only appears in the response.
            type: str
            returned: always
            sample: Succeeded
        scale_set_eviction_policy:
            description:
                - ScaleSetEvictionPolicy to be used to specify eviction policy for Spot virtual machine scale set.
            type: str
            returned: always
            sample: null
        scale_set_priority:
            description:
                - caleSetPriority to be used to specify virtual machine scale set priority.
            type: str
            returned: always
            sample: null
        spot_max_price:
            description:
                - SpotMaxPrice to be used to specify the maximum price you are willing to pay in US Dollars.
            type: float
            returned: always
            sample: null
        security_profile:
            description:
                - The security settings of an agent pool.
            type: complex
            contains:
                enable_secure_boot:
                    description:
                        - The secure boot is diabled or enabled.
                    type: bool
                    returned: always
                    sample: true
                enable_vtpm:
                    description:
                        - The vTPM is diabled or enabled.
                    type: bool
                    returned: always
                    sample: true
        type:
            description:
                - Resource Type.
            type: str
            returned: always
            sample: Microsoft.ContainerService/managedClusters/agentPools
        type_properties_type:
            description:
                - AgentPoolType represents types of an agent pool.
            type: str
            returned: always
            sample: VirtualMachineScaleSets
        upgrade_settings:
            description:
                - Settings for upgrading the agentpool.
            type: str
            returned: always
            sample: null
        vm_size:
            description:
                - Size of agent VMs.
            type: str
            returned: always
            sample: Standard_B2s
        vnet_subnet_id:
            description:
                - VNet SubnetID specifies the VNet's subnet identifier.
            type: str
            returned: always
            sample: null
        kubelet_config:
            description:
                - The Kubelet configuration on the agent pool nodes.
            type: dict
            returned: always
            sample: {
                    cpu_cfs_quota: true,
                    cpu_cfs_quota_period: 100ms,
                    cpu_manager_policy: static,
                    fail_swap_on: false,
                    image_gc_high_threshold: 85,
                    image_gc_low_threshold: 80,
                    topology_manager_policy: none
                }
        linux_os_config:
            description:
                - The OS configuration of Linux agent nodes.
            type: dict
            returned: always
            sample: {
                    swap_file_size_mb: 1500,
                    sysctls: {},
                    transparent_huge_page_defrag: defer+madvise,
                    transparent_huge_page_enabled: madvise
                }
        power_state:
            description:
                - The agent pool's power state.
            type: dict
            returned: always
            sample: {code: Running}
        tags:
            description:
                - The tags of the node agent pool.
            type: dict
            returned: always
            sample: {key1: value1, key2: value2}
        kubelet_disk_type:
            description:
                - Determines the placement of emptyDir volumes, container runtime data root, and Kubelet ephemeral storage.
            type: str
            returned: always
            sample: OS
        workload_runtime:
            description:
                - Determines the type of workload a node can run.
            type: str
            returned: always
            sample: OCIContainer
        scale_down_mode:
            description:
                - This also effects the cluster autoscaler behavior.
            type: str
            returned: always
            sample: Delete
        node_public_ip_prefix_id:
            description:
                - The Azure Public IP prefix's ID.
            type: str
            returned: always
            sample: null
        proximity_placement_group_id:
            description:
                - The ID for Proximity Placement Group.
            type: str
            returned: always
            sample: /subscriptions/xxx-xxx/resourceGroups/myRG/providers/Microsoft.Compute/proximityPlacementGroups/proxi01
        enable_encryption_at_host:
            description:
                - This is only supported on certain VM sizes and in certain Azure regions.
            type: bool
            returned: always
            sample: false
        enable_ultra_ssd:
            description:
                - Whether enable FIPS node pool.
            type: bool
            returned: always
            sample: false
        gpu_instance_profile:
            description:
                - GPUInstanceProfile to be used to specify GPU MIG instance profile for supported GPU VM SKU.
            type: str
            returned: always
            sample: MIG1g
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    pass


class AzureRMAgentPoolInfo(AzureRMModuleBase):
    def __init__(self):
        # define user inputs into argument
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str'
            ),
            cluster_name=dict(
                type='str',
                required=True
            )
        )
        # store the results of the module operation
        self.results = dict()
        self.resource_group = None
        self.name = None
        self.cluster_name = None

        super(AzureRMAgentPoolInfo, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=False, facts_module=True)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec):
            setattr(self, key, kwargs[key])

        if self.name:
            aks_agent_pools = [self.get_agentpool()]
        else:
            aks_agent_pools = self.list_agentpool()
        self.results['aks_agent_pools'] = [self.to_dict(x) for x in aks_agent_pools]
        return self.results

    def get_agentpool(self):
        try:
            return self.managedcluster_client.agent_pools.get(self.resource_group, self.cluster_name, self.name)
        except ResourceNotFoundError:
            pass

    def list_agentpool(self):
        result = []
        try:
            resp = self.managedcluster_client.agent_pools.list(self.resource_group, self.cluster_name)
            while True:
                result.append(resp.next())
        except StopIteration:
            pass
        except Exception:
            pass
        return result

    def to_dict(self, agent_pool):
        if not agent_pool:
            return None
        agent_pool_dict = dict(
            resource_group=self.resource_group,
            cluster_name=self.cluster_name,
            id=agent_pool.id,
            type=agent_pool.type,
            name=agent_pool.name,
            count=agent_pool.count,
            vm_size=agent_pool.vm_size,
            os_disk_size_gb=agent_pool.os_disk_size_gb,
            vnet_subnet_id=agent_pool.vnet_subnet_id,
            max_pods=agent_pool.max_pods,
            os_type=agent_pool.os_type,
            os_sku=agent_pool.os_sku,
            max_count=agent_pool.max_count,
            min_count=agent_pool.min_count,
            enable_auto_scaling=agent_pool.enable_auto_scaling,
            type_properties_type=agent_pool.type_properties_type,
            mode=agent_pool.mode,
            availability_zones=[],
            orchestrator_version=agent_pool.orchestrator_version,
            node_image_version=agent_pool.node_image_version,
            upgrade_settings=dict(),
            provisioning_state=agent_pool.provisioning_state,
            enable_node_public_ip=agent_pool.enable_node_public_ip,
            scale_set_priority=agent_pool.scale_set_priority,
            scale_set_eviction_policy=agent_pool.scale_set_eviction_policy,
            spot_max_price=agent_pool.spot_max_price,
            node_labels=agent_pool.node_labels,
            node_taints=agent_pool.node_taints,
            tags=agent_pool.tags,
            kubelet_disk_type=agent_pool.kubelet_disk_type,
            workload_runtime=agent_pool.workload_runtime,
            scale_down_mode=agent_pool.scale_down_mode,
            power_state=dict(),
            node_public_ip_prefix_id=agent_pool.node_public_ip_prefix_id,
            proximity_placement_group_id=agent_pool.proximity_placement_group_id,
            kubelet_config=dict(),
            linux_os_config=dict(),
            security_profile=dict(),
            enable_encryption_at_host=agent_pool.enable_encryption_at_host,
            enable_ultra_ssd=agent_pool.enable_ultra_ssd,
            enable_fips=agent_pool.enable_fips,
            gpu_instance_profile=agent_pool.gpu_instance_profile,
        )

        if agent_pool.upgrade_settings is not None:
            agent_pool_dict['upgrade_settings']['max_surge'] = agent_pool.upgrade_settings.max_surge
        else:
            agent_pool_dict['upgrade_settings'] = None

        if agent_pool.availability_zones is not None:
            for key in agent_pool.availability_zones:
                agent_pool_dict['availability_zones'].append(int(key))
        else:
            agent_pool_dict['availability_zones'] = None

        if agent_pool.kubelet_config is not None:
            agent_pool_dict['kubelet_config'] = agent_pool.kubelet_config.as_dict()
        else:
            agent_pool_dict['kubelet_config'] = None

        if agent_pool.linux_os_config is not None:
            agent_pool_dict['linux_os_config']['transparent_huge_page_enabled'] = agent_pool.linux_os_config.transparent_huge_page_enabled
            agent_pool_dict['linux_os_config']['transparent_huge_page_defrag'] = agent_pool.linux_os_config.transparent_huge_page_defrag
            agent_pool_dict['linux_os_config']['swap_file_size_mb'] = agent_pool.linux_os_config.swap_file_size_mb
            agent_pool_dict['linux_os_config']['sysctls'] = dict()
            if agent_pool.linux_os_config.sysctls is not None:
                agent_pool_dict['linux_os_config']['sysctls'] = agent_pool.linux_os_config.sysctls.as_dict()
            else:
                agent_pool_dict['linux_os_config']['sysctls'] = None
        else:
            agent_pool_dict['linux_os_config'] = None

        if agent_pool.power_state is not None:
            agent_pool_dict['power_state']['code'] = agent_pool.power_state.code
        else:
            agent_pool_dict['power_state'] = None

        if agent_pool.security_profile is not None:
            agent_pool_dict['security_profile']['enable_vtpm'] = agent_pool.security_profile.enable_vtpm
            agent_pool_dict['security_profile']['enable_secure_boot'] = agent_pool.security_profile.enable_secure_boot
        else:
            agent_pool_dict['security_profile'] = None

        return agent_pool_dict


def main():
    AzureRMAgentPoolInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: wireless_access_points_factory_reset_request_provision_v1
short_description: Resource module for Wireless Access Points Factory Reset Request Provision V1
description:
- Manage operation create of the resource Wireless Access Points Factory Reset Request Provision V1.
- >
   This API is used to factory reset Access Points. It is supported for maximum 100 Access Points per request.
   Factory reset clears all configurations from the Access Points. After factory reset the Access Point may become
   unreachable from the currently associated Wireless Controller and may or may not join back the same controller.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  apMacAddresses:
    description: List of Access Point's Ethernet MAC addresses, set maximum 100 ethernet
      MAC addresses per request.
    elements: str
    type: list
  keepStaticIPConfig:
    description: Set the value of keepStaticIPConfig to false, to clear all configurations
      from Access Points and set the value of keepStaticIPConfig to true, to clear all
      configurations from Access Points without clearing static IP configuration.
    type: bool
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Wireless FactoryResetAccessPointsV1
  description: Complete reference of the FactoryResetAccessPointsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!factory-reset-access-points
notes:
  - SDK Method used are
    wireless.Wireless.factory_reset_access_points_v1,

  - Paths used are
    post /dna/intent/api/v1/wirelessAccessPoints/factoryResetRequest/provision,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.wireless_access_points_factory_reset_request_provision_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    apMacAddresses:
    - string
    keepStaticIPConfig: true

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""
